#define _CAMERA_CPP_
#include "vce.h"

#include <d3d8.h>
#include <d3dx8.h>
#include <dinput.h>
#include "camera.h"
#include "input.h"
#include "common.h"
#include "draw.h"
#include "main.h"
#include "drawearth.h"
#include "obj.h"
#include "drawupoint.h"
#define MOVE_LEN 0.01f
#define MOVE_RAD 0.05f

float cam_rad;

/* -------------------------------------------------------------------------
 * initialize camera
 * -------------------------------------------------------------------------*/
BOOL camera_init( void)
{
    LPDIRECT3DDEVICE8 d3d_dev;
    d3d_dev = d3d_get_d3d_dev();
    /*
    cam_eye_x = 12.0f;
    cam_eye_y = 30.0f;
    cam_eye_z = -30.0f;
    cam_at_x = 12.0f;
    cam_at_y = 0.0f;
    cam_at_z = 6.0f;
    */
    cam_eye_x = 0.0f;
    cam_eye_y = 0.0f;
    cam_eye_z = -4.0f;
    cam_at_x = 0.0f;
    cam_at_y = 0.0f;
    cam_at_z = 0.0f;
    cam_rad = 0;
    /* set a camera of coordinate */
    D3DXMatrixLookAtLH( &mat_view,
                        &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                        &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                        &D3DXVECTOR3( 0,1,0));
    d3d_dev->SetTransform( D3DTS_VIEW, &mat_view);
	return TRUE;
}
#define WAITT (30*5)
static int automove_waitt = 0;
void camera_automove( void)
{
    float n_x, n_y, n_z; /* next eye*/
    static float dx, dy, dz;
    float n_a_x, n_a_y, n_a_z; /* next at */
    static float dax,day,daz;
    int r;
    if( automove_waitt >= WAITT && automove_waitt < WAITT*2) {
        automove_waitt ++;
    }
    else if( automove_waitt >= WAITT*2) {
        n_x = (rand() % 300 + 100) / 100.0f;
        r = rand() % 2;
        if( r) n_x *= -1;
        n_y = (rand() % 300 + 100) / 100.0f;
        r = rand() % 2;
        if( r) n_y *= -1;
        n_z = (rand() % 300 + 100) / 100.0f;
        r = rand() % 2;
        if( r) n_z *= -1;
        n_a_x = (rand() % 100) / 100.0f;
        r = rand() % 2;
        if( r) n_a_x *= -1;
        n_a_y = (rand() % 100) / 100.0f;
        r = rand() % 2;
        if( r) n_a_y *= -1;
        n_a_z = (rand() % 100) / 100.0f;
        r = rand() % 2;
        if( r) n_a_z *= -1;
        dx = (n_x - cam_eye_x ) / (float)WAITT;
        dy = (n_y - cam_eye_y ) / (float)WAITT;
        dz = (n_z - cam_eye_z ) / (float)WAITT;
        dax = (n_a_x - cam_at_x ) / (float)WAITT;
        day = (n_a_y - cam_at_y ) / (float)WAITT;
        daz = (n_a_z - cam_at_z ) / (float)WAITT;
        automove_waitt = 0;
    }
    else {
        cam_eye_x += dx;
        cam_eye_y += dy;
        cam_eye_z += dz;
        cam_at_x += dax;
        cam_at_y += day;
        cam_at_z += daz;
        D3DXMatrixLookAtLH( &mat_view,
                            &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                            &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                            &D3DXVECTOR3( D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32 )));
        d3d_dev->SetTransform( D3DTS_VIEW, &mat_view);
        automove_waitt ++;
    }
    
}
static float to_up_dx,  to_up_dy,  to_up_dz;
static float  to_up_dax, to_up_day, to_up_daz;
#define TO_UP_WAITT (30*2)
/* -------------------------------------------------------------------------
 * 
 * -------------------------------------------------------------------------*/
void camera_move_to_upoint_init( float lo, float la)
{
    float to_x, to_y, to_z,x,y,z;
    float lo_rad = lo / 180.0f * PI - PI/2.0f;
    float la_rad = la / 180.0f * PI;
    D3DXMATRIX rmat;
    
    find_upoint_flg = 1;
    to_x = cosf( la_rad) * cosf( lo_rad);
    to_y = sinf( la_rad);
    to_z = cosf( la_rad) * sinf( lo_rad);
    earth_get_rot_mat( obj_search( "earth"), &rmat );
    
    x = to_x * rmat._11 + to_y * rmat._21 + to_z * rmat._31;
    y = to_x * rmat._12 + to_y * rmat._22 + to_z * rmat._32;
    z = to_x * rmat._13 + to_y * rmat._23 + to_z * rmat._33;
    x *= 4;
    y *= 4-1;
    z *= 4;
    to_up_dx = (x - cam_eye_x ) / (float)TO_UP_WAITT;
    to_up_dy = (y - cam_eye_y ) / (float)TO_UP_WAITT;
    to_up_dz = (z - cam_eye_z ) / (float)TO_UP_WAITT;
    to_up_dax = - cam_at_x / (float)TO_UP_WAITT;
    to_up_day = - cam_at_y / (float)TO_UP_WAITT;
    to_up_daz = - cam_at_z / (float)TO_UP_WAITT;
    automove_waitt = 0;
    
    obj_t *obj;
    upoint_work_t *up;
    for (obj = obj_start; obj != NULL; obj = obj->next) {
        if( obj->type != OBJ_TYPEUPOINT) continue;
        up = (upoint_work_t *)obj->work;
        if( up->hit) up->hit = 0;
    }
}

void camera_move_to_upoint( void)
{
    if( automove_waitt < TO_UP_WAITT) {
        D3DXMATRIX matv;
        cam_eye_x += to_up_dx;
        cam_eye_y += to_up_dy;
        cam_eye_z += to_up_dz;
        cam_at_x += to_up_dax;
        cam_at_y += to_up_day;
        cam_at_z += to_up_daz;
        D3DXMatrixLookAtLH( &mat_view,
                            &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                            &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                            &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32 ));
        d3d_dev->SetTransform( D3DTS_VIEW, &mat_view);
        automove_waitt ++;
    }
    else if( automove_waitt < TO_UP_WAITT*3) {
        automove_waitt ++;
    }
    else {
        find_upoint_flg = 0;
        automove_waitt = WAITT*2;
        obj_t *obj;
        upoint_work_t *up;
        for (obj = obj_start; obj != NULL; obj = obj->next) {
            if( obj->type != OBJ_TYPEUPOINT) continue;
            up = (upoint_work_t *)obj->work;
            if( up->hit) up->hit = 0;
        }
    }
}
    
/* -------------------------------------------------------------------------
 * It moves camera depending on input state.
 * -------------------------------------------------------------------------*/
void camera_move( void)
{
    D3DXMATRIX mat_tmp, mat_rotate;
    D3DXVECTOR3 vec_tmp, vec_tmp2;
    
    /* rotate x,y axis when during press Z key */
    if( input_keyboard_data[DIK_Z]) {
        if( input_mouse_move_x != 0 ) {
            /* calculate z vector of camera */
            D3DXVec3Subtract( &vec_tmp,
                              &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                              &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z));
            /* rotate vector. */
            /* It get y axis vector from view matrix */
            /* rotate along y axis of camera(rotation of optional axis) */
            D3DXMatrixRotationAxis( &mat_rotate,
                                    &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32),
                                    (float)input_mouse_move_x*MOVE_RAD *PI/180.0f);
            D3DXVec3TransformCoord( &vec_tmp2, &vec_tmp, &mat_rotate);
            /* It re-dispositions vector on world coordinate. */
            vec_tmp2.x += cam_at_x;
            vec_tmp2.y += cam_at_y;
            vec_tmp2.z += cam_at_z;
            cam_eye_x = vec_tmp2.x;
            cam_eye_y = vec_tmp2.y;
            cam_eye_z = vec_tmp2.z;
            /* make view */
            D3DXMatrixLookAtLH( &mat_view,
                                &vec_tmp2,
                                &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                                &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32));
            /* reset light */
//            d3d_setlight( mat_view._13, mat_view._23, mat_view._33);
            
        }
        if( input_mouse_move_y != 0 ) {
            /* calculate z axis vector of camera. */
            D3DXVec3Subtract( &vec_tmp,
                              &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                              &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z));
            /* rotate vector. */
            /* It get x axis vector from view matrix */
            /* rotate along x axis of camera(rotation of optional axis) */
            D3DXMatrixRotationAxis( &mat_rotate,
                                    &D3DXVECTOR3( mat_view._11, mat_view._21, mat_view._31),
                                    (float)input_mouse_move_y*MOVE_RAD *PI/180.0f);
            D3DXVec3TransformCoord( &vec_tmp2, &vec_tmp, &mat_rotate);
            /* It re-dispositions vector on world coordinate. */
            vec_tmp2.x += cam_at_x;
            vec_tmp2.y += cam_at_y;
            vec_tmp2.z += cam_at_z;
            cam_eye_x = vec_tmp2.x;
            cam_eye_y = vec_tmp2.y;
            cam_eye_z = vec_tmp2.z;
            
            /* make view */
            D3DXMatrixLookAtLH( &mat_view,
                                &vec_tmp2,
                                &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                                &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32));
            /* reset light */
//            d3d_setlight( mat_view._13, mat_view._23, mat_view._33);
            
            
        }
    }
    /* rotate z axis when during press X key */
    if( input_keyboard_data[DIK_X]) {
        if( input_mouse_move_x != 0 ) {
            /* make a rotation matrix that corresponds view z axis. */
            D3DXMatrixRotationAxis( &mat_rotate,
                                    &D3DXVECTOR3( mat_view._13, mat_view._23, mat_view._33),
                                    (float)input_mouse_move_x*MOVE_RAD *PI/180.0f);
            /* rotate y axis vector */
            D3DXVec3TransformCoord( &vec_tmp2,
                                    &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32)
                                    , &mat_rotate);
            /* make a view.
             * z axis rotation is not changes
             *   point of view or point of at view */
            D3DXMatrixLookAtLH( &mat_view,
                                &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                                &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                                &vec_tmp2);
        }
    }
    /* move x,y axis when during press A Key*/
    if( input_keyboard_data[DIK_A] ) {
        if( input_mouse_move_x != 0 ) {
            float x = input_mouse_move_x * MOVE_LEN;
            float tmp;
            tmp = x * mat_view._11;
            cam_eye_x -= tmp;
            cam_at_x -= tmp;
            tmp = x * mat_view._21;
            cam_eye_y -= tmp;
            cam_at_y -= tmp;
            tmp = x * mat_view._31;
            cam_eye_z -= tmp;
            cam_at_z -= tmp;
            /* make view */
            D3DXMatrixLookAtLH( &mat_view,
                                &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                                &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                                &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32));
        }
        if( input_mouse_move_y != 0 ) {
            float x = input_mouse_move_y * MOVE_LEN;
            float tmp;
            tmp = x * mat_view._12;
            cam_eye_x += tmp;
            cam_at_x += tmp;
            tmp = x * mat_view._22;
            cam_eye_y += tmp;
            cam_at_y += tmp;
            tmp = x * mat_view._32;
            cam_eye_z += tmp;
            cam_at_z += tmp;
            /* make view */
            D3DXMatrixLookAtLH( &mat_view,
                                &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                                &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                                &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32));
        }
    }
    /* move z axis when during press S key */
    if( input_keyboard_data[DIK_S] ) {
        if( input_mouse_move_y != 0 ) {
            float z = input_mouse_move_y * MOVE_LEN;
            float tmpx, tmpy, tmpz;
            D3DXVec3Normalize( &vec_tmp,
                               &D3DXVECTOR3( mat_view._13, mat_view._23, mat_view._33));
            tmpx = z * mat_view._13;
            tmpy = z * mat_view._23;
            tmpz = z * mat_view._33;
//            if( cam_eye_x - tmpx >= cam_at_x &&
//                cam_eye_y - tmpy >= cam_at_y &&
//                cam_eye_z - tmpz >= cam_at_z)
//            {
                cam_eye_x -= tmpx;
                cam_eye_y -= tmpy;
                cam_eye_z -= tmpz;
                
                cam_at_x -= tmpx;
                cam_at_y -= tmpy;
                cam_at_z -= tmpz;
                /* make view*/
                    D3DXMatrixLookAtLH( &mat_view,
                                        &D3DXVECTOR3( cam_eye_x,cam_eye_y,cam_eye_z),
                                        &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                                        &D3DXVECTOR3( mat_view._12, mat_view._22, mat_view._32));
//            }
        }
    }
}    

