#include "vce.h"

#include <windows.h>
#include <commctrl.h>
#include "config.h"
#include "resource.h"
#include "net.h"
#include "common.h"
#include "lumiproto_cli.h"
#include "main.h"
#include "win.h"
#include "drawcn.h"

HWND hwnd_dlg_networking;
HWND hwnd_dlg_miscellaneous;

#define REG_LOCATION "Software\\CommunityEngine\\LuminouSphere"
int config_dlg_no_error;

int check_coordinate1( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str);
int check_coordinate2( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str);
int check_coordinate3( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str);
int check_sphere_rate( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str);
typedef enum {
    CTYPE_CHECK,
    CTYPE_INTVAL,
    CTYPE_STRING,
    CTYPE_CB_STRING, /* combo box string (PENDING)*/
}CONFIG_CONTROL_TYPE;

/* default setting */
config_enum_t config = {
    { "autologin", 1, "", REG_DWORD, IDC_CONFIG_AUTOLOGIN, CTYPE_CHECK, NULL},
    { "screensaver", 0, "", REG_DWORD, IDC_CONFIG_SCREENSAVER, CTYPE_CHECK, NULL},
    { "nickname", 0, "", REG_SZ, IDC_CONFIG_NICKNAME, CTYPE_STRING, NULL}, 
    /* this coordinates is for displaying */
    { "longitude1", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LONGITUDE1, CTYPE_INTVAL, check_coordinate1}, 
    { "longitude2", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LONGITUDE2, CTYPE_INTVAL, check_coordinate2}, 
    { "longitude3", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LONGITUDE3, CTYPE_INTVAL, check_coordinate2}, 
    { "latitude1", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LATITUDE1, CTYPE_INTVAL, check_coordinate3},
    { "latitude2", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LATITUDE2, CTYPE_INTVAL, check_coordinate2},
    { "latitude3", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LATITUDE3, CTYPE_INTVAL, check_coordinate2},
    /* */
    { "longitude", INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LONGITUDE4, CTYPE_INTVAL, NULL},
    { "latitude",  INIT_LONGITUDE, "", REG_DWORD, IDC_CONFIG_LATITUDE4, CTYPE_INTVAL, NULL},
    
    { "color_r", 255, "", REG_DWORD, IDC_CONFIG_COLOR_R, CTYPE_INTVAL, NULL},
    { "color_g", 255, "", REG_DWORD, IDC_CONFIG_COLOR_G, CTYPE_INTVAL, NULL},
    { "color_b", 255, "", REG_DWORD, IDC_CONFIG_COLOR_B, CTYPE_INTVAL, NULL},
    { "sphere_polygon_rate", 30, "", REG_DWORD, IDC_CONFIG_SPHERE_POLYGON_RATE, CTYPE_INTVAL, check_sphere_rate},
    { "low_earth_texture", 0, "", REG_DWORD, IDC_CONFIG_LOWTEX, CTYPE_CHECK, NULL},
#ifdef _DEBUG
    { "host", 0, "civic",REG_SZ, IDC_CONFIG_HOST, CTYPE_STRING, NULL},
#else
    { "host", 0, "sv.luminousphere.net",REG_SZ, IDC_CONFIG_HOST, CTYPE_STRING, NULL},
#endif
    { "port", LUMIPROTO_DEFAULT_FRONTEND_PORT , "", REG_DWORD, IDC_CONFIG_PORT, CTYPE_INTVAL, NULL},
    /* not exist in configration dialog */
    { "last_x", CW_USEDEFAULT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "last_y", CW_USEDEFAULT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "last_width", SCREEN_WIDTH, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "last_height", SCREEN_HEIGHT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "view_wireframe", 0, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "view_rotate", 1, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "view_findroute", 1, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "view_verbose", 1, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "view_automove_viewpoint", 1, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "send_msgdlg_last_x", CW_USEDEFAULT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "send_msgdlg_last_y", CW_USEDEFAULT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "incoming_msgdlg_last_x", CW_USEDEFAULT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    { "incoming_msgdlg_last_y", CW_USEDEFAULT, "", REG_DWORD, -1, CTYPE_INTVAL, NULL},
    
};

#define config_size ( sizeof( config) / sizeof( config_t))

HWND handle_child_dlg[3];
static int child_dlg_cnt;

int check_coordinate1( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str)
{
    if( v < -180 || v > 180) {
        MessageBox( hwnd, "invalid coordinate", EARTH_WINDOWNAME, MB_OK);
        return FALSE;
    }
    return TRUE;
}
int check_coordinate3( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str)
{
    if( v < -90 || v > 90) {
        MessageBox( hwnd, "invalid coordinate", EARTH_WINDOWNAME, MB_OK);
        return FALSE;
    }
    return TRUE;
}
int check_coordinate2( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str)
{
    if( v < 0 || v > 60) {
        MessageBox( hwnd, "invalid coordinate", EARTH_WINDOWNAME, MB_OK);
        return FALSE;
    }
    return TRUE;
}
int check_sphere_rate( HWND hwnd, HWND dlgitem, config_t *c, int v, char *str)
{
    if( v < 0 || v > 1000) {
        MessageBox( hwnd, "invalid sphere polygon rate", EARTH_WINDOWNAME, MB_OK);
        return FALSE;
    }
    return TRUE;
}
// -------------------------------------------------------------------------
// 
// -------------------------------------------------------------------------
LRESULT CALLBACK config_dlgproc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
    NMHDR *nmhdr;
    config_t *c = (config_t *)&config;
    char buff[1024];
    switch( msg) {
    case WM_COMMAND:
        switch( LOWORD( wp)) {
        case IDC_CONFIG_GETCOORDINATE:
            if( !config_get_coordinate_flg){
                cn_init( 255,255,255);
                config_get_coordinate_flg = 1;
            }
            break;
        }
        break;
    case WM_INITDIALOG:
//        hwnd_dlg_global = hwnd;
 
        for( int i = 0; i < config_size; i ++ ) {
            HWND dlgitem;
            if( (c+i)->control_id == -1 ) continue;
            dlgitem = GetDlgItem( hwnd, (c+i)->control_id);
            if( dlgitem == NULL) continue;
            switch( (c+i)->control_type) {
            case CTYPE_CHECK:
                if( (c+i)->int_val) {
                    CheckDlgButton( hwnd, (c+i)->control_id, BST_CHECKED);
                }
                break;
            case CTYPE_INTVAL:
                vce_snprintf( buff, sizeof( buff), "%d", (c+i)->int_val);
                SetWindowText( dlgitem, buff);
                break;
            case CTYPE_STRING:
                SetWindowText( dlgitem, (c+i)->str);
                break;
            case CTYPE_CB_STRING:
                break;
            }
        }
        handle_child_dlg[child_dlg_cnt++] = hwnd;

        config_dlg_no_error = 1;
        return true;
    case WM_NOTIFY:
        nmhdr = ( NMHDR *)lp;
        switch( nmhdr->code) {
        case PSN_APPLY:
            config_dlg_no_error = 1;
            for( int i = 0; i < config_size; i ++ ) {
                int v = 0;
                char buff[1024];
                HWND dlgitem;
                buff[0] = '\0';
                if( (c+i)->control_id == -1 ) continue;
                dlgitem = GetDlgItem( hwnd, (c+i)->control_id);
                if( dlgitem == NULL) continue;
                switch( (c+i)->control_type) {
                case CTYPE_CHECK:
                    if( IsDlgButtonChecked( hwnd, (c+i)->control_id)
                        == BST_CHECKED)
                    {
                        v = 1;
                    }
                    else {
                        v = 0;
                    }
                    break;
                case CTYPE_INTVAL:
                    GetWindowText( dlgitem, buff, sizeof( buff));
                    v = atoi( buff);
                    break;
                case CTYPE_STRING:
                    GetWindowText( dlgitem, buff, sizeof( buff));
                    break;
                case CTYPE_CB_STRING:
                    break;
                }
                int ret = TRUE;
                if( (c+i)->checkfunc) {
                    ret = (c+i)->checkfunc( hwnd, dlgitem, (c+i), v, buff);
                }
                if( ret) {
                    (c+i)->int_val = v;
                    vce_snprintf( (c+i)->str, sizeof( c+i)->str, "%s", buff);
                }
                else {
                    config_dlg_no_error = 0;
                    return false;
                }
            }
            return true;
        case PSN_RESET:
            config_dlg_no_error = 1;
            break;
        default:
            return false;
        }
    }
    return false;
}
int get_config_index_from_control_id( int control_id)
{
    config_t *c = (config_t *)&config;
    for( int i = 0; i < config_size; i ++ ) {
        if( (c+i)->control_id == control_id) return i;
    }
    return -1;
}
// -------------------------------------------------------------------------
// 
// -------------------------------------------------------------------------
int config_dlg_update( int control_id, char *str, int value)
{
    int cindex;
    char buff[1024];
    config_t *c = (config_t *)&config;
    cindex = get_config_index_from_control_id( control_id);
    if( cindex == -1 ) return -1;
    for( int i = 0; i < child_dlg_cnt; i ++ ) {
        HWND dlgitem = GetDlgItem( handle_child_dlg[i], control_id);
        if( dlgitem != NULL ) {
            switch( (c+cindex)->control_type) {
            case CTYPE_CHECK:
                CheckDlgButton( handle_child_dlg[i],
                                (c+cindex)->control_id,
                                (value ? BST_CHECKED: BST_UNCHECKED));
                break;
            case CTYPE_INTVAL:
                vce_snprintf( buff, sizeof( buff), "%d", value);
                SetWindowText( dlgitem, buff);
                break;
            case CTYPE_STRING:
                SetWindowText( dlgitem, str);
                break;
            case CTYPE_CB_STRING:
                break;
            }
            
            return 0;
        }
    }
    return -1;
}
// -------------------------------------------------------------------------
// 
// -------------------------------------------------------------------------
HWND create_config( HWND hwnd, int modal_flg)
{
    HINSTANCE hinst;
    PROPSHEETPAGE psp;
    PROPSHEETHEADER psh;
    HPROPSHEETPAGE hpsp[3];

    memset( handle_child_dlg, 0, sizeof( handle_child_dlg));
    child_dlg_cnt = 0;
    
    hinst = (HINSTANCE)GetWindowLong( hwnd, GWL_HINSTANCE);
    memset( &psp, 0, sizeof( psp));
    psp.dwSize = sizeof( PROPSHEETPAGE);
    psp.dwFlags = PSP_DEFAULT;
    psp.hInstance = hInst;
//    psp.pszTemplate = "IDD_KEYCONFIG_PAGE1";
    psp.pszTemplate = MAKEINTRESOURCE( IDD_PAGE1);
    psp.pfnDlgProc = (DLGPROC)config_dlgproc;
    hpsp[0] = CreatePropertySheetPage( &psp);
    
//    psp.pszTemplate = "IDD_KEYCONFIG_PAGE2";
    psp.pszTemplate = MAKEINTRESOURCE( IDD_PAGE2);
    psp.pfnDlgProc = (DLGPROC)config_dlgproc;
    hpsp[1] = CreatePropertySheetPage( &psp);
    psp.pszTemplate = MAKEINTRESOURCE( IDD_PAGE3);
    psp.pfnDlgProc = (DLGPROC)config_dlgproc;
    hpsp[2] = CreatePropertySheetPage( &psp);
    memset( &psh, 0, sizeof( PROPSHEETHEADER));
    psh.dwSize = sizeof( PROPSHEETHEADER);
    psh.dwFlags = PSH_NOAPPLYNOW ;// | PSH_USECALLBACK;
    if( !modal_flg) {
        psh.dwFlags |= PSH_MODELESS;
    }
    psh.hInstance = hinst;
    psh.hwndParent = hwnd;
    psh.nPages = 3;
    psh.phpage = hpsp;
//    psh.ppsp = (LPCPROPSHEETPAGE) hpsp;
    psh.pszCaption = "LuminouSphere Configuration";
//    psh.pfnCallback = (PFNPROPSHEETCALLBACK)prop_sheet_proc;
    return (HWND)PropertySheet( &psh);
}    

// -------------------------------------------------------------------------
// read from registry
// -------------------------------------------------------------------------
int config_read( void)
{
    LONG lResult;
    DWORD dwDisposition;
    HKEY hKeyResult;
    DWORD dwData;
    DWORD dwType;
    config_t *c = (config_t *)&config;

    lResult = RegCreateKeyEx( HKEY_CURRENT_USER,
                              REG_LOCATION,
                              NULL, "",
                              REG_OPTION_NON_VOLATILE,
                              KEY_ALL_ACCESS, NULL,
                              &hKeyResult, &dwDisposition);
    if( lResult == ERROR_SUCCESS) {
        if( dwDisposition == REG_OPENED_EXISTING_KEY) {
            for( int i = 0; i < config_size; i ++ ) {
                dwType = (c+i)->type;
                LPBYTE lpb;
                if( (c+i)->type == REG_SZ) {
                    dwData = sizeof( (c+i)->str);
                    lpb = (LPBYTE)(c+i)->str;
                } else if( (c+i)->type == REG_DWORD) {
                    dwData = sizeof( (c+i)->int_val);
                    lpb = (LPBYTE)&((c+i)->int_val);
                }
                RegQueryValueEx( hKeyResult, (c+i)->keyname,
                                 NULL, &dwType,
                                 lpb, &dwData);
            }
        }
    }
    RegCloseKey( hKeyResult);
    return 0;
}
// -------------------------------------------------------------------------
// write to registry
// -------------------------------------------------------------------------
int config_write( void)
{
    HKEY hKey;
    RegOpenKeyEx( HKEY_CURRENT_USER,REG_LOCATION,
                  0, KEY_ALL_ACCESS, &hKey);
    config_t *c = (config_t *)&config;
    for( int i = 0; i < config_size; i ++ ) {
        LPBYTE lpb;
        int len;
        if( (c + i)->type == REG_SZ) {
            lpb = (LPBYTE)(c+i)->str;
            len = strlen( (c+i)->str) + 1;
        } else if( (c+i)->type == REG_DWORD) {
            lpb = (LPBYTE)&((c+i)->int_val);
            len = sizeof( (c+i)->int_val);
        }
        RegSetValueEx( hKey, (c+i)->keyname, 0, (c+i)->type,
                       lpb, len);
    }
    RegCloseKey( hKey);
    return 0;
}
