#define _DRAW_CPP_
#include "vce.h"
#include <windows.h>
#include <d3d8.h>
#include <d3dx8.h>
#include "common.h"
#include "draw.h"
#include "main.h"
#include "win.h"
#include "obj.h"
#include "mesh.h"
#include "font.h"
#include "camera.h"
#include "frame.h"
#include "parsecmd.h"
#include "config.h"
#include "msg.h"

LONGLONG draw_polygon_sec_start;

LPDIRECT3D8 d3d = NULL;

D3DPRESENT_PARAMETERS d3d_param;

LPDIRECT3DDEVICE8 d3d_get_d3d_dev( void)
{
    return d3d_dev;
}    
void d3d_setlight( float x, float y, float z)
{
    D3DXVECTOR3 vecDir;
    vecDir = D3DXVECTOR3( x, y, z);
    D3DXVec3Normalize( (D3DXVECTOR3*)&draw_light.Direction, &vecDir);
    d3d_dev->SetLight( 0, &draw_light);
    
}
void create_light( void)
{
    ZeroMemory( &draw_light, sizeof( D3DLIGHT8));
    draw_light.Type = D3DLIGHT_DIRECTIONAL;
//    draw_light.Type = D3DLIGHT_POINT;
    draw_light.Diffuse.r = 1.0f;
    draw_light.Diffuse.g = 1.0f;
    draw_light.Diffuse.b = 1.0f;
    
    draw_light.Range = 100.0f;
    d3d_setlight( 100.0f, 10.0f, -10.0f);
//    d3d_setlight( mat_view._13, mat_view._23, mat_view._33);
    d3d_dev->LightEnable( 0, TRUE);
    d3d_dev->SetRenderState( D3DRS_LIGHTING, TRUE);
    
}
/* -------------------------------------------------------------------------
 * initialize direct 3d
 * -------------------------------------------------------------------------*/
BOOL d3d_init( void)
{
    D3DDISPLAYMODE disp;
    
    d3d = Direct3DCreate8( D3D_SDK_VERSION);
    if( d3d == NULL) {
        return FALSE;
    }
    ZeroMemory( &d3d_param, sizeof( D3DPRESENT_PARAMETERS));
    d3d->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &disp);
//    gScreen_mode = TRUE;
//    if( gScreen_mode) {
    switch( screen_saver_mode) {
    case SMODE_SCR:
//        d3d_param.BackBufferFormat = D3DFMT_R5G6B5;
        d3d_param.BackBufferFormat = disp.Format;
        d3d_param.BackBufferWidth = disp.Width;
        d3d_param.BackBufferHeight = disp.Height;
        window_width = client_area_width = disp.Width;
        window_height = client_area_height = disp.Height;
        break;
    default:
//     }   else{
        d3d_param.BackBufferFormat = disp.Format;
        d3d_param.Windowed = 1;
        d3d_param.BackBufferWidth = config.last_width.int_val;
        client_area_width = config.last_width.int_val;
        d3d_param.BackBufferHeight = config.last_height.int_val;
        client_area_height = config.last_height.int_val;
    }
    d3d_param.SwapEffect = D3DSWAPEFFECT_FLIP;
    d3d_param.EnableAutoDepthStencil = 1;
    d3d_param.AutoDepthStencilFormat = D3DFMT_D16;
    if( FAILED( d3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
                                   hWnd,
                                   D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                   &d3d_param, &d3d_dev)))
    {
        if( FAILED( d3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
                                       hWnd,
                                       D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                       &d3d_param, &d3d_dev)))
        {
            if( FAILED( d3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_REF,
                                           hWnd,
                                           D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                           &d3d_param, &d3d_dev)))
            {
                D3D_RELEASE( d3d);
                return FALSE;
            }
        }
    }
    camera_init();
    create_light();
    
//    D3DXMatrixPerspectiveFovLH( &mat_proj, 60.0f*PI/180.0f, 1.0f, 2.0f, 100.0f);
    D3DXMatrixPerspectiveFovLH( &mat_proj, 30.0f*PI/180.0f, client_area_width/(float)client_area_height,
                                0.005f, 100.0f);
    d3d_dev->SetTransform( D3DTS_PROJECTION, &mat_proj);
    
    // Turn on the zbuffer
    d3d_dev->SetRenderState( D3DRS_ZENABLE, TRUE );

    // Turn on ambient lighting 
    d3d_dev->SetRenderState( D3DRS_AMBIENT, 0x00a0a0a0 );
    draw_polygon_count = 0;
    
    /* texture */
    d3d_dev->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW);
    return TRUE;
}
/* -------------------------------------------------------------------------
 * 
 * -------------------------------------------------------------------------*/
void d3d_finalize( void)
{

	D3D_RELEASE( d3d_dev);
    D3D_RELEASE( d3d);
}
/* -------------------------------------------------------------------------
 *
 * -------------------------------------------------------------------------*/
void draw_obj( void)
{
    obj_t *obj;
    int i;
    
    d3d_dev->SetTransform( D3DTS_VIEW, &mat_view);
    
    for( obj = obj_start; obj != NULL; obj = obj->next) {
        D3DXMATRIX mat_world;
        mesh_t *mesh;
        D3DXMATRIX mat_trans;
//        D3DXMATRIX mat_rotate;
        if( obj->mesh_id < 0 ) continue;
        mesh = mesh_get_mesh_from_id( obj->mesh_id);
        if( mesh == NULL) continue;
        D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
//        D3DXMatrixRotationY( &mat_rotate, timeGetTime()/1000.0f );
//        mat_world = mat_rotate * mat_trans;
        mat_world = mat_trans;
        d3d_dev->SetTransform( D3DTS_WORLD, &mat_world );
        d3d_dev->SetRenderState( D3DRS_WRAP0, 0 );
        for( i = 0; i < mesh->num; i ++ ) {
            d3d_dev->SetMaterial( &mesh->materials[i]);
            d3d_dev->SetTexture( 0, mesh->textures[i]);
            if( mesh->textures[i] != NULL) {
                d3d_dev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE);
                d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
                d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
                d3d_dev->SetTextureStageState( 0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);
                d3d_dev->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
                d3d_dev->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
            }
            mesh->mesh->DrawSubset( i);
        }
        draw_polygon_count += mesh->mesh->GetNumFaces();
    }
}    
/* -------------------------------------------------------------------------
 *
 * -------------------------------------------------------------------------*/
void draw_main( void)
{
    obj_t *obj;
    d3d_dev->Clear( 0, NULL,
                    D3DCLEAR_TARGET |D3DCLEAR_ZBUFFER,
                    D3DCOLOR_XRGB( 0,0,0),
                    1.0f, 0);
    d3d_dev->BeginScene();
    draw_obj();
    for( obj = obj_start; obj != NULL; obj = obj->next) {
        if( obj->drawfunc) {
            obj->drawfunc( obj);
        }
    }
    message_append_to_font_all();
    font_draw();
    d3d_dev->EndScene();
    if( FAILED( d3d_dev->Present( NULL, NULL, NULL, NULL))) {
        draw_restore();
//        d3d_dev->Reset( &d3d_param);
    }
    
}    
/* -------------------------------------------------------------------------
 * 
 * -------------------------------------------------------------------------*/
void draw_calc_polygon_rate( void)
{
    LONGLONG count_current;
    QueryPerformanceCounter((LARGE_INTEGER *)&count_current);
    if( count_current >= draw_polygon_sec_start + freq_perfcount) {
        draw_polygon_rate = draw_polygon_count;
        draw_polygon_count = 0;
        draw_polygon_sec_start = count_current;
    }
}
/* -------------------------------------------------------------------------
 * 
 * -------------------------------------------------------------------------*/
void draw_polygon_set_sec_start( void)
{
    QueryPerformanceCounter((LARGE_INTEGER *)&draw_polygon_sec_start);
}    
void draw_restore( void)
{
    LPDIRECT3DSURFACE8 back_buffer;
    D3DSURFACE_DESC back_buffer_desc;
    RECT rect;
    GetClientRect( hWnd, &rect);
    d3d_param.BackBufferWidth = rect.right - rect.left;
    d3d_param.BackBufferHeight = rect.bottom - rect.top;
    
    if( FAILED(d3d_dev->Reset( &d3d_param))) return;
    font_restore();
    d3d_dev->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &back_buffer);
    back_buffer->GetDesc( &back_buffer_desc);
    back_buffer->Release();
    client_area_width = back_buffer_desc.Width;
    client_area_height = back_buffer_desc.Height;
    message_restore();
    
    d3d_dev->SetTextureStageState( 0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP );
    d3d_dev->SetTextureStageState( 0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP );
    d3d_dev->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
    d3d_dev->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    d3d_dev->SetTextureStageState( 1, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP );
    d3d_dev->SetTextureStageState( 1, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP );
    d3d_dev->SetTextureStageState( 1, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
    d3d_dev->SetTextureStageState( 1, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    d3d_dev->SetTextureStageState( 2, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP );
    d3d_dev->SetTextureStageState( 2, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP );
    d3d_dev->SetTextureStageState( 2, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
    d3d_dev->SetTextureStageState( 2, D3DTSS_MINFILTER, D3DTEXF_LINEAR );

    d3d_dev->SetRenderState( D3DRS_ZENABLE, TRUE );
    D3DXMatrixPerspectiveFovLH( &mat_proj, 30.0f*PI/180.0f,
                                client_area_width/(float)client_area_height,
                                0.005f, 100.0f);
    d3d_dev->SetTransform( D3DTS_PROJECTION, &mat_proj);
    create_light();
    // Turn on ambient lighting 
    d3d_dev->SetRenderState( D3DRS_AMBIENT, 0x00a0a0a0 );
}
