/*
 * cn = cursor needle
 */
#include "vce.h"

#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>

#include "drawearth.h"
#include "drawcn.h"
#include "obj.h"
#include "input.h"
#include "draw.h"
#include "msg.h"
#include "font.h"
#include "m.h"
#include "main.h"
#include "resource.h"
#include "config.h"
#include "win.h"

typedef struct _cn_vertex_t
{
    D3DXVECTOR3 p;
}cn_vertex_t;

typedef struct _cn_work_t {
    unsigned char r,g,b;
    LPDIRECT3DVERTEXBUFFER8 vb;
}cn_work_t;

#define D3DFVF_CNVERTEX (D3DFVF_XYZ)

void cn_draw( obj_t *obj)
{
    D3DXMATRIX earthrotmat;

    font_append( "Please click a coordinate where you are.",
                 0, 50, 0, D3DCOLOR_XRGB( 255,255,0));
    earth_get_rot_mat( obj->parent, &earthrotmat );
    cn_work_t *work = (cn_work_t *)obj->work;

    D3DXMATRIX matWorld, mat_trans;
    D3DXMATRIX rmat;
    
    static D3DXVECTOR3 v;
    if( calc_cursor_intersect( mouse_x, mouse_y, &v) == 0 ) return;
    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixMultiply( &matWorld, &earthrotmat, &mat_trans );
    d3d_dev->SetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixInverse( &rmat, NULL, &earthrotmat);
    
    cn_vertex_t *vtx;
    work->vb->Lock( 0,0, (BYTE**)&vtx, 0 );
    float vx, vy, vz;
    float vx2, vy2, vz2;
    
    vx2 = v.x * rmat._11 + v.y * rmat._21 + v.z * rmat._31;
    vy2 = v.x * rmat._12 + v.y * rmat._22 + v.z * rmat._32;
    vz2 = v.x * rmat._13 + v.y * rmat._23 + v.z * rmat._33;
    v *= 1.2f;
    vx = v.x * rmat._11 + v.y * rmat._21 + v.z * rmat._31;
    vy = v.x * rmat._12 + v.y * rmat._22 + v.z * rmat._32;
    vz = v.x * rmat._13 + v.y * rmat._23 + v.z * rmat._33;
    (*vtx++).p = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );    
    (*vtx++).p = D3DXVECTOR3( vx,vy,vz);
    work->vb->Unlock();
    
    D3DMATERIAL8 material;
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = ((float)work->r) / 256.0f;
    material.Diffuse.g = ((float)work->g) / 256.0f;
    material.Diffuse.b = ((float)work->b) / 256.0f;
    material.Ambient.r = 0.5f + ((float)work->r) / 256.0f * 10.0f;
    material.Ambient.g = 0.5f + ((float)work->g) / 256.0f * 10.0f;
    material.Ambient.b = 0.5f + ((float)work->b) / 256.0f * 10.0f;
    d3d_dev->SetMaterial( &material);
    
    d3d_dev->SetStreamSource( 0, work->vb, sizeof(cn_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_CNVERTEX );
    d3d_dev->DrawPrimitive( D3DPT_LINELIST, 0, 1 );
    float lo,la;
    earth_get_lolah( vx2, vy2, vz2, &lo, &la);
    if( input_mouse_rbutton_new_pressed) {
        obj->destroy_flg = 1;
        config_get_coordinate_flg = 0;
    }
    if( input_mouse_lbutton_new_pressed) {
        if( hWnd_config_dlg) {
            int h,m,s;
            earth_conv_rad_to_hms( lo, &h,&m,&s);
            config_dlg_update( IDC_CONFIG_LONGITUDE1, "", h);
            config_dlg_update( IDC_CONFIG_LONGITUDE2, "", m);
            config_dlg_update( IDC_CONFIG_LONGITUDE3, "", s);
            config_dlg_update( IDC_CONFIG_LONGITUDE4, "", (int)(lo * LUMIPROTO_FIX_POINT));
            earth_conv_rad_to_hms( la, &h,&m,&s);
            config_dlg_update( IDC_CONFIG_LATITUDE1, "", h);
            config_dlg_update( IDC_CONFIG_LATITUDE2, "", m);
            config_dlg_update( IDC_CONFIG_LATITUDE3, "", s);
            config_dlg_update( IDC_CONFIG_LATITUDE4, "", (int)(la * LUMIPROTO_FIX_POINT));
        }
        obj->destroy_flg = 1;
        config_get_coordinate_flg = 0;
    }
}
void cn_finalize( obj_t *obj)
{
    cn_work_t *work = (cn_work_t *)obj->work;
    D3D_RELEASE( work->vb);
}

int cn_init( unsigned char r,unsigned char g,unsigned char b )
                 
{
    obj_t *obj;
    cn_work_t *work;
    char s[100];
    vce_snprintf( s, sizeof(s), "cn");
    obj = obj_new( 0, sizeof( cn_work_t), s );
    if(!obj) return 0;
    work = (cn_work_t *)obj->work;
    obj->drawfunc = cn_draw;
    obj->destroyfunc = cn_finalize;
    work->r = r;
    work->g = g;
    work->b = b;
    obj->parent = obj_search( "earth" );
    if( FAILED( d3d_dev->CreateVertexBuffer(
        2 * sizeof( cn_vertex_t),
        D3DUSAGE_WRITEONLY, D3DFVF_CNVERTEX,
        D3DPOOL_MANAGED, &work->vb ) ) ){
        obj->destroy_flg = TRUE;
        return 0;
    }
    
    return 1;
}
