#define _DRAWEARTH_CPP_
#include "vce.h"
#include <windows.h>
#include <process.h>
#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>

#include "draw.h"
#include "drawearth.h"
#include "drawupoint.h"
#include "obj.h"
#include "config.h"



LPDIRECT3DTEXTURE8 tex_earth;
LPDIRECT3DTEXTURE8 tex_moon;
//int sphere_vertices_num;
//float star_rotate_angle;
typedef struct _star_vertex_t
{
    D3DXVECTOR3 p;
    D3DXVECTOR3 n;
    FLOAT       tu1, tv1;
    FLOAT       tu2, tv2;
}star_vertex_t;

typedef struct _star_vbdata_t {
    D3DXVECTOR3 p;
    D3DXVECTOR3 n;
    FLOAT tu2,tv2;
}star_vbdata_t;

typedef struct _star_work_t {
    float scale;
    int rotate_flg;
    int sphere_rings;
    int sphere_segments;
    float rotate_angle;
    int sphere_vertices_num;
    LPDIRECT3DTEXTURE8 tex;
    LPDIRECT3DVERTEXBUFFER8 vb;
    D3DXMATRIX mat_rotate;
    star_vbdata_t *v;
}star_work_t;


#define D3DFVF_STARVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX2)


#define SPHERE_RINGS 30
#define SPHERE_SEGMENTS 60

void set_vertex_data( obj_t *obj)
{
    star_work_t *work;
    DWORD dwNumSphereRings;
    DWORD dwNumSphereSegments;
    FLOAT fDeltaRingAngle;
    FLOAT fDeltaSegAngle;
    work = (star_work_t *)obj->work;
    dwNumSphereRings = work->sphere_rings;
    dwNumSphereSegments = work->sphere_segments;
    fDeltaRingAngle = ( D3DX_PI / dwNumSphereRings );
    fDeltaSegAngle  = ( 2.0f * D3DX_PI / dwNumSphereSegments );
    
    D3DXVECTOR4 vT;
//    FLOAT fScale;
    int i = 0;

    // Generate the group of rings for the sphere
    for( DWORD ring = 0; ring < dwNumSphereRings; ring++ )
    {
        FLOAT r0 = sinf( (ring+0) * fDeltaRingAngle )*work->scale;
        FLOAT r1 = sinf( (ring+1) * fDeltaRingAngle )*work->scale;
        FLOAT y0 = cosf( (ring+0) * fDeltaRingAngle )*work->scale;
        FLOAT y1 = cosf( (ring+1) * fDeltaRingAngle )*work->scale;

        // Generate the group of segments for the current ring
        for( DWORD seg = 0; seg < (dwNumSphereSegments+1); seg++ )
        {
            FLOAT x0 =  r0 * sinf( seg * fDeltaSegAngle );
            FLOAT z0 =  r0 * cosf( seg * fDeltaSegAngle );
            FLOAT x1 =  r1 * sinf( seg * fDeltaSegAngle );
            FLOAT z1 =  r1 * cosf( seg * fDeltaSegAngle );

            // Add two vertices to the strip which makes up the sphere
            // (using the transformed normal to generate texture coords)
            (work->v + i)->p = (work->v + i)->n   = D3DXVECTOR3(x0,y0,z0);
            (work->v + i)->tu2 = -((FLOAT)seg)/dwNumSphereSegments;
            (work->v + i)->tv2 = (ring+0)/(FLOAT)dwNumSphereRings;
            i++;

            (work->v + i)->p   = (work->v + i)->n   = D3DXVECTOR3(x1,y1,z1);
            (work->v + i)->tu2 = -((FLOAT)seg)/dwNumSphereSegments;
            (work->v + i)->tv2 = (ring+1)/(FLOAT)dwNumSphereRings;
            i++;
        }
    }
}
void draw_vertex_buffer( obj_t *obj, D3DXMATRIX *matWorldView,star_vertex_t *vtx)
{
    star_work_t *work;
    DWORD dwNumSphereRings;
    DWORD dwNumSphereSegments;
    FLOAT fDeltaRingAngle;
    FLOAT fDeltaSegAngle;
    
    work = (star_work_t *)obj->work;
    dwNumSphereRings = work->sphere_rings;
    dwNumSphereSegments = work->sphere_segments;
    fDeltaRingAngle = ( D3DX_PI / dwNumSphereRings );
    fDeltaSegAngle  = ( 2.0f * D3DX_PI / dwNumSphereSegments );
    
    D3DXVECTOR4 vT;
    FLOAT fScale;
    int i = 0;
    // Generate the group of rings for the sphere
    for( DWORD ring = 0; ring < dwNumSphereRings; ring++ )
    {
//        FLOAT r0 = sinf( (ring+0) * fDeltaRingAngle )*work->scale;
//        FLOAT r1 = sinf( (ring+1) * fDeltaRingAngle )*work->scale;
//        FLOAT y0 = cosf( (ring+0) * fDeltaRingAngle )*work->scale;
//        FLOAT y1 = cosf( (ring+1) * fDeltaRingAngle )*work->scale;

        // Generate the group of segments for the current ring
        for( DWORD seg = 0; seg < (dwNumSphereSegments+1); seg++ )
        {
//            FLOAT x0 =  r0 * sinf( seg * fDeltaSegAngle );
//            FLOAT z0 =  r0 * cosf( seg * fDeltaSegAngle );
//            FLOAT x1 =  r1 * sinf( seg * fDeltaSegAngle );
//            FLOAT z1 =  r1 * cosf( seg * fDeltaSegAngle );

            // Add two vertices to the strip which makes up the sphere
            // (using the transformed normal to generate texture coords)
//            (*vtx).p   = (*vtx).n   = D3DXVECTOR3(x0,y0,z0);
            (*vtx).p = (work->v +i)->p;
            (*vtx).n = (work->v +i)->n;
            (*vtx).tu2 = (work->v +i)->tu2;
            (*vtx).tv2 = (work->v +i)->tv2;
//            (*vtx).tu1 = (*vtx).tu2 = -((FLOAT)seg)/dwNumSphereSegments;
//            (*vtx).tv1 = (*vtx).tv2 = (ring+0)/(FLOAT)dwNumSphereRings;
            D3DXVec3Transform( &vT, &(*vtx).n, matWorldView );
            fScale = 1.37f / D3DXVec4Length( &vT );
            (*vtx).tu1 = 0.5f + fScale*vT.x;
            (*vtx).tv1 = 0.5f - fScale*vT.y;
            vtx++;
            i ++;
            
//            (*vtx).p   = (*vtx).n   = D3DXVECTOR3(x1,y1,z1);
//            (*vtx).tu1 = (*vtx).tu2 = -((FLOAT)seg)/dwNumSphereSegments;
//            (*vtx).tv1 = (*vtx).tv2 = (ring+1)/(FLOAT)dwNumSphereRings;
            (*vtx).p = (work->v +i)->p;
            (*vtx).n = (work->v +i)->n;
            (*vtx).tu2 = (work->v +i)->tu2;
            (*vtx).tv2 = (work->v +i)->tv2;
            D3DXVec3Transform( &vT, &(*vtx).n, matWorldView );
            fScale = 1.37f / D3DXVec4Length( &vT );
            (*vtx).tu1 = 0.5f + fScale*vT.x;
            (*vtx).tv1 = 0.5f - fScale*vT.y;
            vtx++;
            i ++;
        }
    }
}


void draw_star( obj_t *obj)
{
    // Get the World-View(WV) matrix set
    D3DXMATRIX matWorld, matView, matWorldView, mat_trans;
    D3DMATERIAL8 material;
    star_work_t *work;
    work = (star_work_t *)obj->work;
    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixMultiply( &matWorld, &work->mat_rotate, &mat_trans );
    d3d_dev->SetTransform( D3DTS_WORLD, &matWorld );
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    d3d_dev->GetTransform( D3DTS_VIEW,  &matView );
    D3DXMatrixMultiply( &matWorldView, &matWorld, &matView );
    
    // Lock the vertex buffer
    star_vertex_t *vtx;
    work->vb->Lock( 0, 0, (BYTE**)&vtx, 0 );

    // Establish constants used in sphere generation
//    DWORD dwNumSphereRings    = m_bHighTesselation ? 15 :  5;
//    DWORD dwNumSphereSegments = m_bHighTesselation ? 30 : 10;
    draw_vertex_buffer( obj, &matWorldView, vtx);
    work->vb->Unlock();
    
    d3d_dev->SetRenderState( D3DRS_WRAP0, D3DWRAP_U | D3DWRAP_V );
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    d3d_dev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1 );
    d3d_dev->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    d3d_dev->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = 1.0f;
    material.Diffuse.g = 1.0f;
    material.Diffuse.b = 1.0f;
    material.Ambient.r = 1.0f;
    material.Ambient.g = 1.0f;
    material.Ambient.b = 1.0f;
    d3d_dev->SetMaterial( &material);
    d3d_dev->SetTexture( 0, work->tex);
    d3d_dev->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 1 );
//    d3d_dev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    
//    d3d_dev->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
    
    d3d_dev->SetStreamSource( 0, work->vb, sizeof( star_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_STARVERTEX );
    
    d3d_dev->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, work->sphere_vertices_num-2 );

    
    if( config.view_wireframe.int_val) {
        d3d_dev->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME);
    }
    else {
        d3d_dev->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID);
    }
    // Restore texture stage states
    d3d_dev->SetTexture( 0, NULL );
    d3d_dev->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    d3d_dev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    d3d_dev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_DISABLE );
    d3d_dev->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
    d3d_dev->SetTextureStageState( 1, D3DTSS_ALPHAOP, D3DTOP_DISABLE );
    d3d_dev->SetTextureStageState( 2, D3DTSS_COLOROP, D3DTOP_DISABLE );
    d3d_dev->SetTextureStageState( 2, D3DTSS_ALPHAOP, D3DTOP_DISABLE );
}

void draw_star_finalize( obj_t *obj)
{
    star_work_t *work;
    work = (star_work_t *)obj->work;
    free( work->v);
    D3D_RELEASE( work->tex);
    D3D_RELEASE( work->vb);
}
void star_rotate( obj_t *obj)
{
    static int prev_time = 0;
    int now;
    star_work_t *work;
    work = (star_work_t *)obj->work;
    
    now = GetTickCount();
    if( config.view_rotate.int_val && work->rotate_flg) {
        work->rotate_angle += (( now - prev_time) / 20000.0f);
    }
    prev_time = now;
    D3DXMATRIX m;
    

    D3DXMatrixRotationY( &work->mat_rotate, - work->rotate_angle );
    D3DXMatrixRotationX( &m, - 0.3f );
    D3DXMatrixMultiply( &work->mat_rotate, &work->mat_rotate, &m );
    
}
int star_vbdata_init( obj_t *obj)
{
    star_work_t *work;
    work = (star_work_t *)obj->work;
    work->v = (star_vbdata_t *)malloc( sizeof( star_vbdata_t) * work->sphere_vertices_num);
    if( work->v == NULL) return FALSE;
    set_vertex_data( obj);
    return TRUE;
}
int draw_earth_init( void)
{
    obj_t *obj;
    star_work_t *work;
    obj = obj_new( 0, sizeof( star_work_t), "earth" );
    obj->drawfunc = draw_star;
    obj->destroyfunc = draw_star_finalize;
    obj->motionfunc = star_rotate;
    work = (star_work_t *)obj->work;
    work->rotate_angle = 0.0f;

#if 0
    if( FAILED( D3DXCreateTextureFromFileEx(
        d3d_dev, "earth_middle.bmp",
        D3DX_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,0, D3DFMT_R5G6B5,
        D3DPOOL_MANAGED,D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,
        D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,0, NULL, NULL, &work->tex)))
    {
        return 0;
    }
#endif
    work->tex = tex_earth;
    work->sphere_rings = config.sphere_polygon_rate.int_val;
    work->sphere_segments = config.sphere_polygon_rate.int_val*2;
    work->sphere_vertices_num = 2 * work->sphere_rings * ( work->sphere_segments+1);
    if( FAILED( d3d_dev->CreateVertexBuffer( work->sphere_vertices_num
                                             * sizeof( star_vertex_t),
                                             D3DUSAGE_WRITEONLY, D3DFVF_STARVERTEX,
                                             D3DPOOL_MANAGED, &work->vb ) ) )
    {
        return 0;
    }
    work->rotate_flg = 1;
    work->scale = 1;
    star_vbdata_init( obj);
    return 1;
}
int moon_init( void)
{
    obj_t *obj;
    star_work_t *work;
    obj = obj_new( 0, sizeof( star_work_t), "moon");
    obj->drawfunc = draw_star;
    obj->destroyfunc = draw_star_finalize;
    obj->motionfunc = star_rotate;

    work = (star_work_t *)obj->work;
    work->rotate_angle = 0.0f;
    
#if 0
    if( FAILED( D3DXCreateTextureFromFileEx(
        d3d_dev, "moon.bmp",
        D3DX_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,0, D3DFMT_R5G6B5,
        D3DPOOL_MANAGED,D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,
        D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,0, NULL, NULL, &work->tex)))
    {
        return 0;
    }
#endif
    work->tex = tex_moon;
    work->sphere_rings = SPHERE_RINGS / 2;
    work->sphere_segments = SPHERE_SEGMENTS / 2;
    work->sphere_vertices_num = 2 * work->sphere_rings * ( work->sphere_segments+1);
    
    if( FAILED( d3d_dev->CreateVertexBuffer( work->sphere_vertices_num
                                             * sizeof( star_vertex_t),
                                             D3DUSAGE_WRITEONLY, D3DFVF_STARVERTEX,
                                             D3DPOOL_MANAGED, &work->vb ) ) )
    {
        return 0;
    }
    work->rotate_flg = 0;
    work->scale = 0.24f;
    obj->x = 5;
    obj->y = 0;
    obj->z = 1;
    star_vbdata_init( obj);
    return 1;
}

void earth_get_rot_mat( obj_t *e, D3DXMATRIX *m )
{
    star_work_t *sw = (star_work_t*)e->work;
    memcpy( m, &sw->mat_rotate, sizeof(D3DXMATRIX));
}

unsigned __stdcall read_texture_thread( void * param)
{
    char fn[1024];
    if( config.low_earth_texture.int_val) {
        vce_snprintf( fn, sizeof( fn), "earth_small.bmp");
    }
    else {
        vce_snprintf( fn, sizeof( fn), "earth_middle.bmp");
    }
    if( FAILED( D3DXCreateTextureFromFileEx(
        d3d_dev, fn,
        D3DX_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,0, D3DFMT_R5G6B5,
        D3DPOOL_MANAGED,D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,
        D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,0, NULL, NULL, &tex_earth)))
    {
        _endthreadex(0);
        return 0;
    }
    if( FAILED( D3DXCreateTextureFromFileEx(
        d3d_dev, "moon.bmp",
        D3DX_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,0, D3DFMT_R5G6B5,
        D3DPOOL_MANAGED,D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,
        D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,0, NULL, NULL, &tex_moon)))
    {
        _endthreadex(0);
        return 0;
    }
    _endthreadex(0);
    return 0;
}
