#define _DRAWGRID_CPP_

#include "vce.h"


#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>

#include "draw.h"
#include "drawearth.h"
#include "obj.h"


typedef struct _grid_vertex_t
{
    D3DXVECTOR3 p;
}grid_vertex_t;

typedef struct _grid_work_t {
    unsigned char r,g,b;
    LPDIRECT3DVERTEXBUFFER8 vb;
}grid_work_t;


#define D3DFVF_GRIDVERTEX (D3DFVF_XYZ)

#define WORLD_LEN 20
#define WORLD_LEN2 (WORLD_LEN/2)

void grid_draw( obj_t *obj)
{

    grid_work_t *work = (grid_work_t *)obj->work;

    D3DXMATRIX matWorld, matView, matWorldView, mat_trans;
    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->SetTransform( D3DTS_WORLD, &mat_trans );

    grid_vertex_t *vtx;
    work->vb->Lock( 0,0, (BYTE**)&vtx, 0 );
    D3DXVECTOR3 v;
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, -WORLD_LEN2, -WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, -WORLD_LEN2, -WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, WORLD_LEN2, WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, -WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, WORLD_LEN2, WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, -WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, -WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2,  -WORLD_LEN2, -WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2,  WORLD_LEN2, -WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, -WORLD_LEN2, WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2,  -WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, WORLD_LEN2, WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2,  WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, -WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, -WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, -WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, -WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( -WORLD_LEN2, WORLD_LEN2, WORLD_LEN2);
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, WORLD_LEN2, -WORLD_LEN2 );    
    (*vtx++).p = D3DXVECTOR3( WORLD_LEN2, WORLD_LEN2, WORLD_LEN2);
    work->vb->Unlock();

    
    D3DMATERIAL8 material;
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = ((float)work->r) / 256.0f;
    material.Diffuse.g = ((float)work->g) / 256.0f;
    material.Diffuse.b = ((float)work->b) / 256.0f;
    material.Ambient.r = 0.5f + ((float)work->r) / 256.0f * 10.0f;
    material.Ambient.g = 0.5f + ((float)work->g) / 256.0f * 10.0f;
    material.Ambient.b = 0.5f + ((float)work->b) / 256.0f * 10.0f;
    d3d_dev->SetMaterial( &material);
    
    d3d_dev->SetStreamSource( 0, work->vb, sizeof(grid_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_GRIDVERTEX );
    d3d_dev->DrawPrimitive( D3DPT_LINELIST, 0, 12 );
}

void grid_finalize( obj_t *obj)
{
    grid_work_t *work = (grid_work_t *)obj->work;
    D3D_RELEASE( work->vb);
}
int grid_init( void )
                 
{
    obj_t *obj;
    grid_work_t *work;
    char s[100];
    vce_snprintf( s, sizeof(s), "grid" );
    obj = obj_new( 0, sizeof( grid_work_t), s );
    if(!obj) return 0;
    work = (grid_work_t *)obj->work;
    obj->drawfunc = grid_draw;
    obj->destroyfunc = grid_finalize;
    work->r = 255;
    work->g = 255;
    work->b = 255;
    if( FAILED( d3d_dev->CreateVertexBuffer(
        2 * sizeof( grid_vertex_t) * 12,
        D3DUSAGE_WRITEONLY, D3DFVF_GRIDVERTEX,
        D3DPOOL_MANAGED, &work->vb ) ) ){
        obj->destroy_flg = TRUE;
        return 0;
    }
    
    return 1;
}

