#define _DRAWMSG_CPP_


/*
  
  drawmsg.cpp : Handle flying messages
  
 */

#include "vce.h"


#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>
#include <time.h>

#include "draw.h"
#include "drawearth.h"
#include "drawupoint.h"
#include "drawmsg.h"
#include "drawroute.h"
#include "game_main.h"

#include "m.h"
#include "obj.h"
#include "math.h"
#include "dijk.h"
#include "config.h"
#include "msg.h"
#include "main.h"
#include "net.h"
#include "resource.h"
#include "win.h"
#include "incoming.h"

typedef struct _msg_vertex_t
{
    D3DXVECTOR3 p;
} msg_vertex_t;

typedef struct _msg_work_t
{
    unsigned int serial;
    float lo,la; /* from upoint */
    float from_lo, from_la;
    float dest_lo, dest_la;
    float rt_lo, rt_la;  /* router location */
    float t,dt; // intermediary variable
    
    float h_km; // height
    unsigned char r,g,b;
    LPDIRECT3DVERTEXBUFFER8 vb;

    time_t start_time;
    int myflg;
    /* route info */
    int rt[DIJK_MAXP];
    int nexthop; /* indicates next rouer.
                    starts with 3, if [ 43, 32, 22, 55] route */
    char str[LUMIPROTO_MSG_MAX+1];
    char name[LUMIPROTO_NAME_MAX+1];
} msg_work_t;

#define D3DFVF_MSGVERTEX (D3DFVF_XYZ)

#define MSG_LINES 8   /* a message consists of how many lines ? */


void msg_draw( obj_t *obj )
{
    D3DXMATRIX earthrotmat;
    earth_get_rot_mat( obj->parent, &earthrotmat);

    msg_work_t *work = (msg_work_t*) obj->work;

    D3DXMATRIX matWorld, matView, matWorldView, mat_trans;    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixMultiply( &matWorld, &earthrotmat, &mat_trans );
    d3d_dev->SetTransform( D3DTS_WORLD, &matWorld );

    /* movement of the message */
    work->t += work->dt;
    float dlo = (work->rt_lo - work->from_lo);
    float dla = (work->rt_la - work->from_la);
    float cur_lo = work->from_lo + dlo * work->t;
    float cur_la = work->from_la + dla * work->t;

    if( work->t <= 1.0f ){
        obj->destroy_flg = FALSE;
    } else {
        work->nexthop --;
        if( work->nexthop >= 0 ){
            /* re-calc movement */
            get_router_lola( work->rt[work->nexthop],
                             &work->rt_lo, &work->rt_la );
            get_router_lola( work->rt[work->nexthop+1],
                             &work->from_lo, &work->from_la );
            if( work->from_lo < -90.0f && work->rt_lo > 90.0f ){
                work->from_lo += 360.0f;
            } else if( work->from_lo > 90.0f && work->rt_lo < -90.0f ){
                work->rt_lo += 360.0f;
            }
            work->t = 0.0f;
            work->dt = 1.0f / distance( work->from_lo, work->from_la,
                                        work->rt_lo, work->rt_la );
            return;
        } else if( work->nexthop == -1 ){
            /* Finished? */
            get_router_lola( work->rt[0],
                             &work->from_lo, &work->from_la );
            work->rt_lo = work->dest_lo;
            work->rt_la = work->dest_la;
            if( work->from_lo < -90.0f && work->rt_lo > 90.0f ){
                work->from_lo += 360.0f;
            } else if( work->from_lo > 90.0f && work->rt_lo < -90.0f ){
                work->rt_lo += 360.0f;
            }
            
            work->t = 0.0f;
            work->dt = 1.0f / distance( work->from_lo, work->from_la,
                                        work->rt_lo, work->rt_la );
            return;
        } else {
            if( work->myflg) {
                if( screen_saver_mode) {
                    message_append( work->str, work->r, work->g, work->b);
                }
                if( incoming_msg_num == 0 ) {
                    EnableMenuItem( hMenu, ID_INCOMINGMESSAGE, MF_ENABLED);
                    /* If doesn't, Menu appearance isn't reflected immediately. */
                    DrawMenuBar( hWnd); 
                }
                incoming_append( work->serial, work->lo, work->la, work->str, work->name);
            }
            obj->destroy_flg = TRUE;
        }
    }
    
    /* set up vertices */
    msg_vertex_t *vtx;
    work->vb->Lock(0,0, (BYTE**)&vtx, 0 );
    D3DXVECTOR3 v1;
    earth_get_v_lolah( &v1, cur_lo, cur_la, work->h_km );
    for(int i=0;i<MSG_LINES;i++){
        (*vtx++).p = v1;
        (*vtx++).p = random_sphere_vec( v1,
                                        ((float)(rand()%100))/8000.0f );
    }
#if 0    
    for(int i=0;i<MSG_LINES;i++){
        float t = (float)i / (float)MSG_LINES;
        float nt = (float)(i+1)/(float)MSG_LINES;

        D3DXVECTOR3 v2;
        earth_get_v_lolah( &v1, work->from_lo + dlo * t,
                           work->from_la + dla * t,
                           work->h_km );
        earth_get_v_lolah( &v2, work->from_lo + dlo * nt,
                           work->from_la + dla * nt,
                           work->h_km );
        (*vtx++).p = v1;
        (*vtx++).p = v2;
    }
#endif    
    work->vb->Unlock();

    
    
    D3DMATERIAL8 material;
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = ((float)work->r) / 256.0f;
    material.Diffuse.g = ((float)work->g) / 256.0f;
    material.Diffuse.b = ((float)work->b) / 256.0f;
    material.Ambient.r = 0.5f + (((float)work->r) / 256.0f) * 10.0f;
    material.Ambient.g = 0.5f + (((float)work->g) / 256.0f) * 10.0f;
    material.Ambient.b = 0.5f + (((float)work->b) / 256.0f) * 10.0f;
    d3d_dev->SetMaterial( &material);
    
    d3d_dev->SetStreamSource( 0, work->vb, sizeof(msg_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_MSGVERTEX);
    d3d_dev->DrawPrimitive( D3DPT_LINELIST, 0, MSG_LINES );
}

void msg_finalize( obj_t *obj )
{
    msg_work_t *work = (msg_work_t*) obj->work;
    D3D_RELEASE(work->vb);
}

int msg_init( obj_t *from_u, obj_t *to_u, obj_t *parent, char *str, int myflg, char *name )
{
    obj_t *obj;
    char s[100];
    float from_lo, from_la, dest_lo, dest_la; /* dest user */
    msg_work_t *work;
    
    upoint_get_lola( from_u, &from_lo, &from_la );
    upoint_get_lola( to_u, &dest_lo, &dest_la );
    
    vce_snprintf( s, sizeof(s), "msg_%f_%f_%f_%f",
                  from_lo, from_la, dest_lo, dest_la );

    obj = obj_new( 0, sizeof(msg_work_t), s );
    if(!obj)return 0;
    obj->drawfunc = msg_draw;
    obj->destroyfunc = msg_finalize;
    obj->parent = parent;
    work = (msg_work_t*)obj->work;
    work->from_lo = from_lo;
    work->from_la = from_la;
    work->dest_lo = dest_lo;
    work->dest_la = dest_la;
    work->myflg = myflg;
    work->lo = from_lo;
    work->la = from_la;
    work->serial = ((upoint_work_t *)from_u->work)->serial;
    vce_snprintf( work->str, sizeof( work->str), "%s", str);
    vce_snprintf( work->name, sizeof( work->str), "%s", name);
    /* initialize route, and go to the first router */
    float dummy;
    if( config.view_findroute.int_val) {
        find_route( work->rt, ELEMENTOF(work->rt),
                    from_lo, from_la, dest_lo, dest_la, &dummy );
        /* first router! */
        work->nexthop = get_last_route_index(work->rt);
        get_router_lola( work->rt[work->nexthop],
                         &work->rt_lo, &work->rt_la );
    }
    else {
        work->rt_lo = dest_lo;
        work->rt_la = dest_la;
    }
        
    if( work->from_lo > 90.0f && work->rt_lo < -90.0f ){
        work->rt_lo += 360.0f;
    } else if( work->from_lo < -90.0f && work->rt_lo > 90.0f ){
        work->from_lo += 360.0f;
    }
    work->t = 0.0f;

    float aho = distance( work->from_lo, work->from_la,
                          work->rt_lo, work->rt_la );
    work->dt = 1.0f / aho;

    
    upoint_get_height( from_u, &work->h_km );

    upoint_get_color( from_u, &work->r,&work->g,&work->b );

    if( FAILED( d3d_dev->CreateVertexBuffer(
        MSG_LINES * 2 * 2 * sizeof( msg_vertex_t ),
        D3DUSAGE_WRITEONLY, D3DFVF_MSGVERTEX,
        D3DPOOL_MANAGED, &work->vb ))){
        obj->destroy_flg = TRUE;
        return 0;
    }

    work->start_time = time(NULL);

    
    return 1;
}
