#define _DRAWROUTE_CPP_


/*
  drawroute.cpp : Handle routing information and show it

  $Revision: 1.5 $
  $Date: 2002/08/06 14:57:56 $
  $Id: drawroute.cpp,v 1.5 2002/08/06 14:57:56 mas Exp $
                   *
                   |
                   |
                   |
  * -------------- * -------------- *
    \
     \
      \
       \
        *

        * ... router
        | ... route

        router : Similar to user's needle
        route : Lines drawn between router and router

  
*/



#include "vce.h"

#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>
#include <time.h>


#include "game_main.h"
#include "draw.h"
#include "drawearth.h"
#include "drawmsg.h"
#include "drawroute.h"
#include "lumiproto_cli.h"
#include "m.h"
#include "dijk.h"
#include "obj.h"

/* Represents internet router */
typedef struct _router_t
{
    int id;
    float longtitude, latitude; /* location of the router */
} router_t;


/* Represents route information */
typedef struct _rti_t
{
    int id1, id2;
} rti_t;

typedef struct _router_vertex_t
{
    D3DXVECTOR3 p;
} router_vertex_t;
typedef struct _rti_vertex_t
{
    D3DXVECTOR3 p;
} rti_vertex_t;

#define ROUTER_DIV_N 12

typedef struct _router_work_t
{
    int id;
    float longtitude, latitude;
    float h_km;
    unsigned char r,g,b;
    D3DXVECTOR3 v[ROUTER_DIV_N];
    LPDIRECT3DVERTEXBUFFER8 vb;
} router_work_t;

#define RTI_DIV_N 20

typedef struct _rti_work_t
{
    obj_t *rt1obj, *rt2obj;
    LPDIRECT3DVERTEXBUFFER8 vb;
    D3DXVECTOR3 v1[RTI_DIV_N];
    D3DXVECTOR3 v2[RTI_DIV_N];
} rti_work_t;


#define D3DFVF_ROUTERVERTEX (D3DFVF_XYZ)
#define D3DFVF_RTIVERTEX (D3DFVF_XYZ)


#define MAX_ROUTER 200
#define MAX_RTI 1000



int router_mi = -1;
int rti_mi = -1;

dijk_t route_dijk;

void router_draw( obj_t *obj )
{
    D3DXMATRIX earthrotmat;

    earth_get_rot_mat( obj->parent, &earthrotmat );
    router_work_t *work = (router_work_t *)obj->work;

    D3DXMATRIX matWorld, matView, matWorldView, mat_trans;
    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixMultiply( &matWorld, &earthrotmat, &mat_trans );
    d3d_dev->SetTransform( D3DTS_WORLD, &matWorld );

    router_vertex_t *vtx;
    work->vb->Lock( 0,0, (BYTE**)&vtx, 0 );
    D3DXVECTOR3 topv;
    earth_get_v_lolah( &topv, work->longtitude, work->latitude, work->h_km );
    

    for(int i=0;i<ROUTER_DIV_N;i++){
//        float rad = (PI*2.0f)/ ((float)ROUTER_DIV_N) * ((float)i);
//        float d_lo = 0.5f * cosf(rad);
//        float d_la = 0.5f * sinf(rad);

//        D3DXVECTOR3 v;
//        earth_get_v_lolah( &v,
//                           work->longtitude + d_lo,
//                           work->latitude + d_la,
//                           300.0f );
        (*vtx++).p = topv;
        (*vtx++).p = work->v[i];
    }
    work->vb->Unlock();

    D3DMATERIAL8 material;
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = ((float)work->r) / 256.0f;
    material.Diffuse.g = ((float)work->g) / 256.0f;
    material.Diffuse.b = ((float)work->b) / 256.0f;
    material.Ambient.r = 0.5f + ((float)work->r) / 256.0f * 10.0f;
    material.Ambient.g = 0.5f + ((float)work->g) / 256.0f * 10.0f;
    material.Ambient.b = 0.5f + ((float)work->b) / 256.0f * 10.0f;
    d3d_dev->SetMaterial( &material);

    d3d_dev->SetStreamSource( 0, work->vb, sizeof(router_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_ROUTERVERTEX );
    d3d_dev->DrawPrimitive( D3DPT_LINELIST, 0, ROUTER_DIV_N );    
}
void rti_draw( obj_t *obj )
{
    D3DXMATRIX earthrotmat;

    earth_get_rot_mat( obj->parent, &earthrotmat );
    rti_work_t *work = (rti_work_t *)obj->work;


    D3DXMATRIX matWorld, matView, matWorldView, mat_trans;
    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixMultiply( &matWorld, &earthrotmat, &mat_trans );
    d3d_dev->SetTransform( D3DTS_WORLD, &matWorld );

    rti_vertex_t *vtx;

    
    work->vb->Lock( 0,0, (BYTE**) &vtx,0 );
    for(int i=0;i<RTI_DIV_N;i++){
        (*vtx++).p = work->v1[i];
        (*vtx++).p = work->v2[i];
    }
    
    
    work->vb->Unlock();

    D3DMATERIAL8 material;
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = 3.0f;
    material.Diffuse.g = 1.0f;
    material.Diffuse.b = 0.0f;
    material.Ambient.r = 3.0f;
    material.Ambient.g = 1.0f;
    material.Ambient.b = 0.0f;
    d3d_dev->SetMaterial( &material);

    d3d_dev->SetStreamSource( 0, work->vb, sizeof(rti_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_RTIVERTEX );
    d3d_dev->DrawPrimitive( D3DPT_LINELIST, 0, RTI_DIV_N );
    
}
void rti_finalize( obj_t *obj )
{
    rti_work_t *work = (rti_work_t*) obj->work;
    D3D_RELEASE( work->vb);
}
static void init_one_router( router_t *r, obj_t *parent )
{
    char s[100];
    vce_snprintf( s, sizeof(s), "router_%d", r->id );
    
    obj_t *obj = obj_new( 0, sizeof( router_work_t), s );
    if(!obj)return;

    router_work_t *work;
    work = (router_work_t*)obj->work;
    work->id = r->id;
    work->longtitude = r->longtitude;
    work->latitude = r->latitude;
    work->h_km = 500.0f;
    obj->drawfunc = router_draw;
    obj->destroyfunc = router_finalize;
    obj->parent = parent;
    work->r = 200;
    work->g = 0;
    work->b = 0;
    
    for(int i=0;i<ROUTER_DIV_N;i++){
        float rad = (PI*2.0f)/ ((float)ROUTER_DIV_N) * ((float)i);
        float d_lo = 0.3f * cosf(rad);
        float d_la = 0.3f * sinf(rad);

        D3DXVECTOR3 v;
        earth_get_v_lolah( &v,
                           work->longtitude + d_lo,
                           work->latitude + d_la,
                           300.0f );
        work->v[i] = v;
    }
    if( FAILED( d3d_dev->CreateVertexBuffer(
        ROUTER_DIV_N * 2  * sizeof( router_vertex_t),
        D3DUSAGE_WRITEONLY, D3DFVF_ROUTERVERTEX,
        D3DPOOL_MANAGED, &work->vb ) ) ){
        obj->destroy_flg = TRUE;
        return;
    }
}

static void init_one_rti(  rti_t *r, obj_t *parent )
{
    char s[100];
    vce_snprintf( s, sizeof(s), "rti_%d_%d", r->id1, r->id2 );

    obj_t *obj = obj_new( 0, sizeof(rti_work_t), s );
    if(!obj)return;

    rti_work_t *work;
    work = (rti_work_t*)obj->work;
    work->rt1obj = router_search( r->id1 );
    work->rt2obj = router_search( r->id2 );
    if( work->rt1obj == NULL || work->rt2obj == NULL ){
        OutputDebugString( "Bad routing info!!\n");
        return;
    }
    obj->drawfunc = rti_draw;
    obj->destroyfunc = rti_finalize;
    obj->parent = parent;

    router_work_t *rt1w = (router_work_t*) work->rt1obj->work;
    router_work_t *rt2w = (router_work_t*) work->rt2obj->work;
    
    for(int i=0;i<RTI_DIV_N;i++){
        float t = 1.0f / ((float)RTI_DIV_N) * ((float)i);
        float nt = 1.0f / ((float)RTI_DIV_N) * ((float)(i+1));
        /* Check if two routers are facing against dateline */
        float dlo;
        if( rt1w->longtitude > 90.0f && rt2w->longtitude < -90.0f ){
            dlo = (rt2w->longtitude+360.0f) - rt1w->longtitude;
        } else if( rt1w->longtitude < -90.0f && rt2w->longtitude > 90.0f ){
            dlo = rt2w->longtitude - (rt1w->longtitude+360.0f);
        } else {
            dlo = rt2w->longtitude - rt1w->longtitude;
        }
        float dla = rt2w->latitude - rt1w->latitude;
        D3DXVECTOR3 v1, v2;
        earth_get_v_lolah( &v1,
                           rt1w->longtitude + dlo * t,
                           rt1w->latitude + dla * t,
                           rt1w->h_km );
        earth_get_v_lolah( &v2,
                           rt1w->longtitude + dlo * nt,
                           rt1w->latitude + dla * nt,
                           rt1w->h_km );
        work->v1[i] = v1;
        work->v2[i] = v2;
    }
    if( FAILED( d3d_dev->CreateVertexBuffer(
        RTI_DIV_N * 2 * sizeof( rti_vertex_t),
        D3DUSAGE_WRITEONLY, D3DFVF_RTIVERTEX,
        D3DPOOL_MANAGED, &work->vb ) ) ){
        obj->destroy_flg = TRUE;
        return;
    }        
                            
}

void router_finalize( obj_t *obj )
{
    router_work_t *work = (router_work_t*) obj->work;
    D3D_RELEASE( work->vb);
}

    
/*
  Format:

  router=ID,LO,LA

  rti=ID,ID
  
 */
static int readcallb( char *fn, int l, char *p, char *v )
{
    if( strcmp( p, "router" ) == 0 ){
        int id,lo,la;
        if( vce_text_split_index_get_int( v, ',', 0, &id )<0
            || vce_text_split_index_get_int( v, ',', 1, &lo )<0
            || vce_text_split_index_get_int( v, ',', 2, &la )<0 ){
            return -1;
        }
        router_add( id,
                    ((float)lo) / (float)LUMIPROTO_FIX_POINT,
                    ((float)la) / (float)LUMIPROTO_FIX_POINT );
    } else if( strcmp( p, "rti" ) == 0 ){
        int id1,id2;
        if( vce_text_split_index_get_int( v, ',', 0, &id1 )<0
            || vce_text_split_index_get_int( v, ',', 1, &id2 )<0){
            return -1;
        }
        rti_add( id1, id2 );
    }

    return 0;
}




int route_init( char *path )
{
    router_mi = vce_init_array( sizeof(router_t), MAX_ROUTER, "router_a");
    if( router_mi < 0 ){
        return 0;
    }
    rti_mi = vce_init_array( sizeof(rti_t), MAX_RTI, "rti_a" );
    if( rti_mi < 0 ){
        vce_end_array( router_mi );
        return 0;
    }

    vce_read_simple_config_file( path, readcallb );
    {
        router_t *cur;
        ARRAY_SCAN_TYPE( router_mi, cur, router_t *){
            obj_t *eo = obj_search( "earth" );
            if(eo){
                init_one_router(cur, eo);
            }
        }
    }
    {
        rti_t *cur;
        ARRAY_SCAN_TYPE( rti_mi, cur, rti_t *){
            obj_t *eo = obj_search( "earth" );
            if(eo){
                init_one_rti(cur,eo);
            }
        }
    }
    /* Then initialize dijk lib */
    {
        dijk_init( &route_dijk );
        rti_t *cur;
        ARRAY_SCAN_TYPE( rti_mi, cur, rti_t * ){
            float lo1, lo2, la1, la2;
            get_router_lola( cur->id1, &lo1, &la1 );
            get_router_lola( cur->id2, &lo2, &la2 );
            if( lo1 > 90.0f && lo2 < -90.0f ){
                lo2 += 360.0f;
            } else if( lo1 < -90.0f && lo2 > 90.0f ){
                lo1 += 360.0f;
            } 
            float cost = /*sphere_*/ distance( lo1, la1, lo2, la2);//, 0.64f );
            dijk_add_mutual_path( &route_dijk, cur->id1, cur->id2, cost );
            printd( "COST from %d to %d = %f\n", cur->id1, cur->id2, cost );
        }
    }
    return 1;
}

/* Add a router, before adding route info */
void router_add( int id, float lo, float la )
{
    router_t *r;

    r = (router_t*) vce_alloc_array_object( router_mi );
    if(!r){
        OutputDebugString( "cannot add router.\n");
        return;
    }

    r->id = id;
    r->longtitude = lo;
    r->latitude = la;
    
    return;
}
obj_t *router_search( int id )
{
    char s[100];
    vce_snprintf( s, sizeof(s), "router_%d", id );
    return obj_search( s );
}

void rti_add( int id1, int id2 )
{
    rti_t *r;
    r = (rti_t*) vce_alloc_array_object( rti_mi );
    if(!r){
        OutputDebugString( "cannot add rti\n" );
        return;
    }

    r->id1 = id1;
    r->id2 = id2;    
}


/*
  Find the nearest router from one user

  return router ID number.

  lo : longtitude of the user
  la : latitude of the user
  
 */
int find_nearest_router( float lo, float la )
{
    router_t *cur;
    float mind = 9999999999999999.9f;
    int ans_id = -1;
    ARRAY_SCAN_TYPE( router_mi, cur, router_t * ){
        float d;
        if( lo > 90.0f && cur->longtitude < -90.0f ){
            d = /* sphere_*/ distance( cur->longtitude + 360.0f,
                                 cur->latitude,
                                 lo,
                                       la );//, 0.64f  );
        } else if( lo < -90.0f && cur->longtitude > 90.0f ){
            d = /* sphere_*/ distance( cur->longtitude,
                                 cur->latitude,
                                 lo + 360.0f,
                                       la );//, 0.64f );
        } else {
            d = /* sphere_*/ distance( cur->longtitude, cur->latitude,
                                       lo, la ); //, 0.64f );
        }
        if( d < mind ){
            ans_id = cur->id;
            mind = d;
        }
    }
    return ans_id;
}
void get_router_lola( int id, float *lo, float *la )
{
    obj_t *o = router_search(id);
    if(!o)return;

    router_work_t *work = (router_work_t*)o->work;
    
    *lo = work->longtitude;
    *la = work->latitude;
}

/*
  
  int *rt : output route array
  int num : max size of rt
  
 */
void find_route( int *rt, int num, float from_lo, float from_la,
                 float to_lo, float to_la, float *cost )
{
    int from_rt_id = find_nearest_router( from_lo, from_la );
    int to_rt_id = find_nearest_router( to_lo, to_la );
    
    if( from_rt_id<0 || to_rt_id<0 ) return;
    dijk_get_route( &route_dijk, from_rt_id, to_rt_id, rt, num, cost );
}


int get_last_route_index( int *rt )
{
    int ans =-1;
    for(int i=0;;i++){
        if( rt[i] == -1 ){
            break;
        } else {
            ans = i;
        }
    }
    return ans;
}

