#define _DRAWUPOINT_CPP_

#include "vce.h"


#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>

#include "draw.h"
#include "drawearth.h"
#include "obj.h"
#include "m.h"
#include "input.h"
#include "font.h"
#include "main.h"
#include "net.h"
#include "drawupoint.h"

typedef struct _upoint_vertex_t
{
    D3DXVECTOR3 p;
}upoint_vertex_t;


obj_t *my_upoint = NULL;

#define D3DFVF_UPOINTVERTEX (D3DFVF_XYZ)

#define FLASH_CNT 1

void upoint_draw( obj_t *obj)
{
    D3DXMATRIX earthrotmat;

    earth_get_rot_mat( obj->parent, &earthrotmat );
    upoint_work_t *work = (upoint_work_t *)obj->work;
    if( (select_upoint_mode || find_upoint_flg) && work->hit) {
        work->anim_cnt ++;
        if( work->anim_cnt > FLASH_CNT ) {
            work->anim_cnt = 0;
            return;
        }
    }
    D3DXMATRIX matWorld, matView, matWorldView, mat_trans;
    
    D3DXMatrixTranslation( &mat_trans, obj->x, obj->y, obj->z);
    d3d_dev->GetTransform( D3DTS_WORLD, &matWorld );
    D3DXMatrixMultiply( &matWorld, &earthrotmat, &mat_trans );
    d3d_dev->SetTransform( D3DTS_WORLD, &matWorld );



    upoint_vertex_t *vtx;
    work->vb->Lock( 0,0, (BYTE**)&vtx, 0 );
    D3DXVECTOR3 v;
    earth_get_v_lolah( &v, work->longitude, work->latitude, work->h_km );
    (*vtx++).p = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );    
    (*vtx++).p = v;
    work->vb->Unlock();

    
    D3DMATERIAL8 material;
    memset( &material, 0, sizeof( material));
    material.Diffuse.r = ((float)work->r) / 256.0f;
    material.Diffuse.g = ((float)work->g) / 256.0f;
    material.Diffuse.b = ((float)work->b) / 256.0f;
    material.Ambient.r = 0.5f + ((float)work->r) / 256.0f * 10.0f;
    material.Ambient.g = 0.5f + ((float)work->g) / 256.0f * 10.0f;
    material.Ambient.b = 0.5f + ((float)work->b) / 256.0f * 10.0f;
    d3d_dev->SetMaterial( &material);
    
    d3d_dev->SetStreamSource( 0, work->vb, sizeof(upoint_vertex_t));
    d3d_dev->SetVertexShader( D3DFVF_UPOINTVERTEX );
    d3d_dev->DrawPrimitive( D3DPT_LINELIST, 0, 1 );
}

void upoint_finalize( obj_t *obj)
{
    upoint_work_t *work = (upoint_work_t *)obj->work;
    D3D_RELEASE( work->vb);
}
int upoint_init( unsigned int serial,
                 char *name, float lo, float la, float h_km, obj_t *parent,
                 unsigned char r,
                 unsigned char g,
                 unsigned char b )
                 
{
    obj_t *obj;
    upoint_work_t *work;
    char s[100];
//    vce_snprintf( s, sizeof(s), "upoint_%f_%f", lo,la );
    vce_snprintf( s, sizeof(s), "upoint_%u", serial );
    obj = obj_new( 0, sizeof( upoint_work_t), s );
    if(!obj) return 0;
    work = (upoint_work_t *)obj->work;
    work->longitude = lo;
    work->latitude = la;
    work->h_km = h_km;
    work->hit = 0;
    work->anim_cnt = 0;
    vce_snprintf( work->name, sizeof( work->name), "%s", name);
    
    float lo_rad = lo / 180.0f * PI - PI/2.0f;
    float la_rad = la / 180.0f * PI;
    work->sx = cosf( la_rad) * cosf( lo_rad);
    work->sy = sinf( la_rad);
    work->sz = cosf( la_rad) * sinf( lo_rad);
    
    obj->drawfunc = upoint_draw;
    obj->destroyfunc = upoint_finalize;
    obj->parent = parent;
    obj->type = OBJ_TYPEUPOINT;
    work->r = r;
    work->g = g;
    work->b = b;
    work->serial = serial;
    if( serial == my_serial) my_upoint = obj;
    if( FAILED( d3d_dev->CreateVertexBuffer(
        2 * sizeof( upoint_vertex_t),
        D3DUSAGE_WRITEONLY, D3DFVF_UPOINTVERTEX,
        D3DPOOL_MANAGED, &work->vb ) ) ){
        obj->destroy_flg = TRUE;
        return 0;
    }
    
    return 1;
}

void upoint_clear( unsigned int serial)
{
    char s[100];
//    vce_snprintf( s, sizeof(s), "upoint_%f_%f", lo,la);
    vce_snprintf( s, sizeof(s), "upoint_%u", serial);
    obj_delete_by_id( s );
}
void upoint_allclear( void)
{
    obj_delete_by_type( OBJ_TYPEUPOINT);
}
void upoint_get_lola( obj_t *o, float *lo, float *la )
{

    upoint_work_t *uw = (upoint_work_t*) o->work;    
    *lo = uw->longitude;
    *la = uw->latitude;
}

void upoint_get_height( obj_t *o, float *h_km )
{
    upoint_work_t *uw = (upoint_work_t*) o->work;    
    *h_km = uw->h_km;
}

void upoint_get_color( obj_t *o,
                       unsigned char *r,
                       unsigned char *g,
                       unsigned char *b )
{
    upoint_work_t *uw = (upoint_work_t*) o->work;    
    *r = uw->r;
    *g = uw->g;
    *b = uw->b;
    
}
void find_nearest_upoint_from_cursor( void)
{
    D3DXVECTOR3 v;
    obj_t *obj, *next;
    upoint_work_t *work;
    float mind = 9999999999999999.9f;
    obj_t *result = NULL;
    float d;
    D3DXMATRIX earthrotmat;
    D3DXMATRIX rmat;
    if( calc_cursor_intersect( mouse_x, mouse_y, &v) == 0) return;
    earth_get_rot_mat( obj_search( "earth"), &earthrotmat );
    D3DXMatrixInverse( &rmat, NULL, &earthrotmat);
    float vx, vy, vz;
    vx = v.x * rmat._11 + v.y * rmat._21 + v.z * rmat._31;
    vy = v.x * rmat._12 + v.y * rmat._22 + v.z * rmat._32;
    vz = v.x * rmat._13 + v.y * rmat._23 + v.z * rmat._33;
    
    for (obj = obj_start; obj != NULL; obj = next) {
        next = obj->next;
        if( obj->type != OBJ_TYPEUPOINT) continue;
        work = (upoint_work_t *)obj->work;
        /* calculate distance */
        d = (float)sqrt( (work->sx - vx) * (work->sx - vx)
                         + (work->sy - vy) * (work->sy - vy)
                         + (work->sz - vz) * (work->sz - vz));
        work->hit = 0;
        if( d > 0.26f) continue;
        if( d < mind) {
            result = obj;
            mind = d;
        }
    }
    if( result != NULL) {
        work = (upoint_work_t *)result->work;
        work->hit = 1;
    }
}
obj_t *find_upoint_was_selected( void)
{
    obj_t *obj;
    for (obj = obj_start; obj != NULL; obj = obj->next) {
        if( obj->type != OBJ_TYPEUPOINT) continue;
        if( ((upoint_work_t *)obj->work)->hit) return obj;
    }
    return NULL;
}
obj_t *find_upoint_from_serial( unsigned int serial)
{
    char s[100];
    vce_snprintf( s, sizeof(s), "upoint_%u", serial);
    return obj_search( s);    
}
