#include <windows.h>
#include <d3d8.h>
#include <d3dx8.h>

#include "common.h"
#include "font.h"
#include "draw.h"

font_t font_array[FONT_MAX];

font_t font_sort_array[FONT_MAX];
font_cache_t font_cache[FONT_MAX];
LPD3DXFONT d3d_font;


BOOL font_init( void)
{
    int i;
    HDC text_dc = NULL;
    HFONT handle_font = NULL, handle_font_old = NULL;
    LPDIRECT3DDEVICE8 d3d_dev = d3d_get_d3d_dev();
    
    for( i = 0; i < FONT_MAX; i ++ ) {
        font_array[i].use = FALSE;
        font_array[i].str[0] = '\0';
        font_array[i].x = 0;
        font_array[i].y = 0;
        font_array[i].priority = 0;
        font_array[i].color = 0;
    }
    text_dc = CreateCompatibleDC( NULL);
    handle_font = CreateFont( FONT_SIZE_X, FONT_SIZE_Y, 0, 0,
                              FW_REGULAR, FALSE, FALSE, FALSE,
                              SHIFTJIS_CHARSET,
                              OUT_DEFAULT_PRECIS,
                              CLIP_DEFAULT_PRECIS,
                              DEFAULT_QUALITY,
                              DEFAULT_PITCH,
                              "FixedSys");
    if( handle_font == NULL) return FALSE;
    handle_font_old = (HFONT)SelectObject( text_dc, handle_font);
    if( FAILED( D3DXCreateFont( d3d_dev, handle_font, &d3d_font))) {
        return FALSE;
    }
    SelectObject( text_dc, handle_font_old);
    DeleteDC( text_dc);
    DeleteObject( handle_font);
    return TRUE;
}
void font_reset( void)
{
    int i;
    for( i = 0; i < FONT_MAX; i ++ ) {
        font_array[i].use = FALSE;
    }
}    
void font_finalize( void)
{
    d3d_font->Release();
}    
font_t *font_get_font_empty( void)
{
    int i;
    for( i = 0; i < FONT_MAX; i ++ ) {
        if( font_array[i].use == FALSE) return &font_array[i];
    }
    return NULL;
}    
BOOL font_append( char *str, int x, int y, int priority, D3DCOLOR color)
{
    font_t *font;
    font = font_get_font_empty();
    if( font == NULL) return FALSE;
    strcpy( font->str, str);
    font->use = TRUE;
    font->x = x;
    font->y = y;
    font->priority = priority;
    font->color = color;

    
    return TRUE;
}
static int font_cmp_func( const font_t *a, const font_t *b)
{
    if( a->use) {
        if( !b->use) return 1;
        else {
            return a->priority - b->priority;
        }
    }
    else if( b->use) return -1;
    return 0;
}    
void font_draw( void)
{
    int i;
    RECT rect;
    qsort( font_array, FONT_MAX, sizeof( font_t),
          (int(*)(const void *, const void *))font_cmp_func);
    for( i = 0; i < FONT_MAX; i ++ ) {
        if( font_array[i].use) {
            rect.left = font_array[i].x;
            rect.right = strlen( font_array[i].str)*48;
            rect.top = font_array[i].y;
            rect.bottom = 480;
            d3d_font->DrawText( font_array[i].str, -1, &rect,
                                DT_LEFT | DT_EXPANDTABS,
                                font_array[i].color);
            font_array[i].use = FALSE;
                                
        }
    }
    
}    
    
void font_restore( void)
{
    font_finalize();
    font_init();
}
