#include "vce.h"
#include <process.h>

#include <stdio.h>
#include <stdarg.h>
#include <dinput.h>
#include "state.h"
#include "obj.h"
#include "game_main.h"
#include "font.h"
#include "draw.h"
#include "frame.h"
#include "input.h"
#include "camera.h"
#include "main.h"
#include "win.h"
#include "mesh.h"
#include "drawearth.h"
#include "drawroute.h"
#include "drawupoint.h"
#include "dijk.h"
#include "net.h"
#include "drawroute.h"
#include "config.h"
#include "drawgrid.h"
#include "lumiproto_cli.h"
#include "drawnowl.h"
#include "drawspace.h"
#include "msg.h"
#include "sendmsg.h"
#include "incoming.h"

struct D3DVERTEX
{
    D3DXVECTOR3 p;
    D3DXVECTOR3 n;
    FLOAT       tu, tv;
};

/* structure camera of remembering */
struct memory_camera {
    BOOL use;
    float eye_x, eye_y, eye_z;
    float at_x, at_y, at_z;
    float y_x, y_y, y_z;
};
struct memory_camera mem_cam[4];

char net_state_str[NETSTATE_NUM][128] = {
    "not connect",
    "connecting",
    "established",
    "attempt login",
    "login"
};

void print_net_state( void);
void print_verbose( void);

void memory_camera( int mode)
{
    if( mode < 0 || mode >= arraysizeof( mem_cam)) return;
    mem_cam[mode].eye_x = cam_eye_x;
    mem_cam[mode].eye_y = cam_eye_y;
    mem_cam[mode].eye_z = cam_eye_z;
    mem_cam[mode].at_x = cam_at_x;
    mem_cam[mode].at_y = cam_at_y;
    mem_cam[mode].at_z = cam_at_z;
    mem_cam[mode].y_x = mat_view._12;
    mem_cam[mode].y_y = mat_view._22;
    mem_cam[mode].y_z = mat_view._32;
    mem_cam[mode].use = TRUE;
    
}    
void set_camera( int mode)
{
    if( mode < 0 || mode >= arraysizeof( mem_cam)) return;
    if( mem_cam[mode].use == FALSE) return;
    cam_eye_x = mem_cam[mode].eye_x;
    cam_eye_y = mem_cam[mode].eye_y;
    cam_eye_z = mem_cam[mode].eye_z;
    cam_at_x = mem_cam[mode].at_x;
    cam_at_y = mem_cam[mode].at_y;
    cam_at_z = mem_cam[mode].at_z;
    /* make view */
    D3DXMatrixLookAtLH( &mat_view,
                        &D3DXVECTOR3( cam_eye_x, cam_eye_y, cam_eye_z),
                        &D3DXVECTOR3( cam_at_x, cam_at_y, cam_at_z),
                        &D3DXVECTOR3( mem_cam[mode].y_x,mem_cam[mode].y_y,mem_cam[mode].y_z));
    
}
void login_user( void)
{
#if 0
    if( config.longitude1.int_val < 0 || config.longitude1.int_val > 999
        || config.longitude2.int_val < 0 || config.longitude2.int_val > 999
        || config.longitude3.int_val < 0 || config.longitude3.int_val > 999
        || config.latitude1.int_val < 0 || config.latitude1.int_val > 999
        || config.latitude2.int_val < 0 || config.latitude2.int_val > 999
        || config.latitude3.int_val < 0 || config.latitude3.int_val > 999
        )
#endif
    if( config.longitude.int_val == INIT_LONGITUDE ||
        config.latitude.int_val == INIT_LONGITUDE)
    {
        MessageBox( hWnd,
                    "Please input the latitude and longitude in which you live.",
                    EARTH_WINDOWNAME, MB_OK);
        if( !hWnd_config_dlg) {
            hWnd_config_dlg = create_config( hWnd, 0);
        }
    }
    else {
#if 0
        my_longitude = (int)((((float)config.longitude1.int_val) +
            ( ((float)config.longitude2.int_val) / 60.0f ) +
            ( ((float)config.longitude3.int_val) / 60.0f / 60.0f )) * LUMIPROTO_FIX_POINT);
        my_latitude = (int)((((float)config.latitude1.int_val) +
            (((float)config.latitude2.int_val) / 60.0f) +
            (((float)config.latitude3.int_val) / 60.0f / 60.0f)) * LUMIPROTO_FIX_POINT);
#endif
        my_longitude = config.longitude.int_val;
        my_latitude = config.latitude.int_val;
        
        printd( "My= %d,%d", my_longitude, my_latitude );
    }
}
void print_window_title( void)
{
    static int prev_netstat;
    static int prev_user_num;
    static int prev_incoming_msg_num;
    char buff[256];
    if( prev_netstat != netstat || prev_user_num != user_num || prev_incoming_msg_num != incoming_msg_num) {
        if( netstat < 0 || netstat >= NETSTATE_NUM) return;
        vce_snprintf( buff, sizeof( buff),
                      "%s -%s- user:%d incoming:%d",
                      EARTH_WINDOWNAME,
                      net_state_str[netstat],
                      user_num, incoming_msg_num);
        SetWindowText( hWnd, buff);
        prev_netstat = netstat;
        prev_user_num = user_num;
        prev_incoming_msg_num = incoming_msg_num;
    }
}

static HANDLE thread_handle;
void game_main( void)
{
    int sub_state = state_get_sub();
    int i;
    unsigned int dwThreadId;
    DWORD ret;
    switch( sub_state) {
    case SUBSTATE_DUMMY:
        break;
    case SUBSTATE_GAME_MAIN_WAIT_HEAVY_PROC_INIT:
        window_menu_update();
        draw_nowloding_init();
        thread_handle = (HANDLE)_beginthreadex( NULL, 0, read_texture_thread,
                                                NULL,0,&dwThreadId);
        state_set_sub( SUBSTATE_GAME_MAIN_WAIT_HEAVY_PROC_LOOP);
        break;    
    case SUBSTATE_GAME_MAIN_WAIT_HEAVY_PROC_LOOP:
        draw_nowloding();
        ret = WaitForSingleObject( thread_handle, 0);
        if( ret == WAIT_OBJECT_0) {
            state_set_sub( SUBSTATE_GAME_MAIN_INIT);
            CloseHandle( thread_handle);
            break;
        }
        if( ret == WAIT_TIMEOUT) break;
        MessageBox( hWnd, "Fatal error",EARTH_WINDOWNAME, MB_OK|MB_ICONSTOP);
        state_set_sub( SUBSTATE_DUMMY);
        break;
    case SUBSTATE_GAME_MAIN_INIT:
        select_upoint_mode = 0;
        find_upoint_flg = 0;
        for( i = 0; i < arraysizeof( mem_cam); i ++ ) {
            memset( &mem_cam[i], 0, sizeof( mem_cam[i]));
        }
        if( !draw_earth_init()) {
            MessageBox( hWnd, "Couldn't init texture",
                        EARTH_WINDOWNAME, MB_OK| MB_ICONSTOP);
            return;
        }
        if( !moon_init( )) {
            MessageBox( hWnd, "Couldn't init texture",
                        EARTH_WINDOWNAME, MB_OK| MB_ICONSTOP);
            return;
        }
        if( !route_init( "route.txt" )) {
            MessageBox( hWnd, "Couldn't init route",
                        EARTH_WINDOWNAME, MB_OK| MB_ICONSTOP);
            return;
        }
//        if( !grid_init()) {
//            MessageBox( hWnd, "Couldn't init grid",
//                        EARTH_WINDOWNAME, MB_OK| MB_ICONSTOP);
//            return;
//        }
        if( !net_init()) {
            MessageBox( hWnd, "Couldn't init network",
                        EARTH_WINDOWNAME, MB_OK | MB_ICONSTOP );
            return;
        }
        if( !message_init()) {
            MessageBox( hWnd, "Couldn't init message",
                        EARTH_WINDOWNAME, MB_OK | MB_ICONSTOP );
            return;
        }
        space_init();
        if( config.autologin.int_val) {
            login_user();
        }
        state_set_sub( SUBSTATE_GAME_MAIN_LOOP);
        break;
    case SUBSTATE_GAME_MAIN_LOOP:
        net_poll();
        if( config.view_verbose.int_val) {
            print_net_state();
            print_verbose();
        }
        camera_move();
        if( input_keyboard_data_new_pressed[DIK_1]) {
            memory_camera( 0);
        }
        if( input_keyboard_data_new_pressed[DIK_2]) {
            memory_camera( 1);
        }
        if( input_keyboard_data_new_pressed[DIK_3]) {
            memory_camera( 2);
        }
        if( input_keyboard_data_new_pressed[DIK_4]) {
            memory_camera( 3);
        }
        
        if( input_keyboard_data_new_pressed[DIK_F1]) {
            set_camera( 0);
        }
        if( input_keyboard_data_new_pressed[DIK_F2]) {
            set_camera( 1);
        }
        if( input_keyboard_data_new_pressed[DIK_F3]) {
            set_camera( 2);
        }
        if( input_keyboard_data_new_pressed[DIK_F4]) {
            set_camera( 3);
        }
#if 0
        if( input_keyboard_data_new_pressed[DIK_F5]){
            obj_t *eo = obj_search( "earth" );
            upoint_init(0,
                (float)(rand() % 180),
                (float)(rand() % 180),
                500.0f, eo, 255,255,255 );
        }
#endif
        if( input_keyboard_data_new_pressed[DIK_F7]){
            int rt[DIJK_MAXP];
            float cost = 12345678.0f;
            find_route( rt, ELEMENTOF(rt),
                        -155.0f, 40.0f,
                        135.0f, 30.0f,
                        &cost );
            for(int i=0;i<ELEMENTOF(rt);i++){
                printd( "RT %d %d cost=%f", i, rt[i], cost );
            }
        }
        if( find_upoint_flg) {
            camera_move_to_upoint();
        }
        else {
            if( config.view_automove_viewpoint.int_val) {
                camera_automove();
            }
        }
        if( input_mouse_lbutton_new_pressed) {
            if( select_upoint_mode) {
                obj_t *obj;
                obj = find_upoint_was_selected();
                if( obj) {
                    sendmsg_create_dialog(
                        ((upoint_work_t *)obj->work)->serial);
                    select_upoint_mode = 0;
                    break;
                }
            }
        }
        if( input_mouse_rbutton_new_pressed) {
            if( select_upoint_mode) select_upoint_mode = 0;
        }
            
        if( select_upoint_mode) {
            font_append( "Please choose user to whom you want to send a message.",
                         0, 50, 0, D3DCOLOR_XRGB( 255,255,0));
            find_nearest_upoint_from_cursor();
        }
        if( incoming_msg_num > 0) {
            font_append( "Incoming Message!", 0, 70, 0, D3DCOLOR_XRGB( 255,255,0));
        }
        print_window_title();
        break;
    }
}    


void printd( char *fmt, ... )
{
    char mbuf[8192];
    va_list ap;

    va_start( ap,fmt );
    vsprintf( mbuf, fmt, ap );
    va_end(ap);

    OutputDebugString( mbuf );
    OutputDebugString( "\n" );

}
void print_net_state( void)
{
    char buff[128];
    if( netstat < 0 || netstat >= NETSTATE_NUM) return;
    sprintf( buff, "status: %s", net_state_str[netstat]);
    font_append( buff, 0, 10, 0, D3DCOLOR_XRGB( 255,255,0));
}
void print_verbose( void)
{
    char buff[128];
    sprintf( buff, "%3d fps %4u pt %4u dt %d user",
             frame_rate, current_proc_time, current_draw_time, user_num);
    font_append( buff, 0, 30, 0, D3DCOLOR_XRGB( 255,255,255));
    
}


