#include "vce.h"
#include <windows.h>
#include "obj.h"
#include "incoming.h"
#include "msg.h"
#include "msgdlg.h"
#include "resource.h"
#include "net.h"
#include "main.h"
#include "win.h"
#include "drawupoint.h"
#include "camera.h"
#include "config.h"
#include "m.h"

static incoming_list_t *incoming_top;
static incoming_list_t *incoming_bottom;
int incoming_msg_num;

int incoming_init( void)
{
    incoming_msg_num = 0;
    incoming_top = NULL;
    incoming_bottom = NULL;
    return 1;
}
int incoming_append( unsigned int serial, float lo, float la, char *msg, char *name)
{
    incoming_list_t *m_list;
    
    m_list = ( incoming_list_t *)malloc( sizeof( incoming_list_t));
    if( m_list == NULL) {
        return -1;
    }
    strcpy( m_list->msg, msg);
    strcpy( m_list->name, name);
    m_list->serial = serial;
    m_list->longitude = lo;
    m_list->latitude = la;
    m_list->next = NULL;
    if( incoming_top == NULL ) {
        incoming_top = m_list;
        incoming_bottom = m_list;
    }
    else {
        incoming_bottom->next = m_list;
        incoming_bottom = m_list;
    }
    incoming_msg_num ++;
    
    return 0;
}
void incoming_dialog_close( HWND hwnd)
{
    DestroyWindow( hwnd);
    msgdlg_free_one( hwnd);
}
LRESULT CALLBACK incoming_dlg_proc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
    switch( msg) {
    case WM_INITDIALOG:
        return FALSE;
    case WM_COMMAND:
        switch( LOWORD( wp)) {
        case IDOK:
        {
            char buff[256];
            unsigned int serial;
            GetWindowText( GetDlgItem( hwnd, IDC_EDITSENDINFO1), buff, sizeof( buff));
            serial = atoi( buff);
            if( find_upoint_from_serial( serial) == NULL) {
                vce_snprintf( buff, sizeof( buff), "Not found user:%u", serial);
                message_append( buff, 255,255,0);
            }
            else {
                GetWindowText( GetDlgItem( hwnd, IDC_EDITMSG), buff, sizeof( buff));
                net_sendmsg( serial, buff);
            }
            incoming_dialog_close( hwnd);
        }
        case IDCANCEL:
            incoming_dialog_close( hwnd);
            break;
        case IDC_FIND:
        {
            obj_t *obj;
            char buff[256];
            unsigned int serial;
            upoint_work_t *up;
            GetWindowText( GetDlgItem( hwnd, IDC_EDITSENDINFO1), buff, sizeof( buff));
            serial = atoi( buff);
            obj = find_upoint_from_serial( serial);
            if( obj == NULL) break;
            up = (upoint_work_t *)obj->work;
            camera_move_to_upoint_init( up->longitude, up->latitude);
            up->hit = 1;
            break;
        }
        }
        break;
    case WM_MOVE:
        config.incoming_msgdlg_last_x.int_val = LOWORD( lp);
        config.incoming_msgdlg_last_y.int_val = HIWORD( lp);
        break;
        
    case WM_CLOSE:
        incoming_dialog_close( hwnd);
        break;
    default:
        return FALSE;        
    }
    return TRUE;
}

/* -------------------------------------------------------------------------
 * 
 * -------------------------------------------------------------------------*/
int incoming_create_dialog( void)
{
    msg_dlg_t *s;
    char buff[128];
    
    if( incoming_top == NULL) return -1;
    s = (msg_dlg_t *)vce_alloc_array_object( msg_dlg_mi);
    if( s == NULL) return -1;
    s->hwnd = CreateDialog(
        hInst, MAKEINTRESOURCE( IDD_INCOMINGMSG),
        hWnd,(DLGPROC)incoming_dlg_proc);
    
    vce_snprintf( buff, sizeof( buff), "%u", incoming_top->serial);
    SetWindowText( GetDlgItem( s->hwnd, IDC_EDITSENDINFO1), buff);
    SetWindowText( GetDlgItem( s->hwnd, IDC_EDITINCOMINGMSG), incoming_top->msg);
    SetWindowText( GetDlgItem( s->hwnd, IDC_EDITSENDINFO4), incoming_top->name);
    int h,m,sec;
    earth_conv_rad_to_hms( incoming_top->longitude, &h, &m, &sec);
    vce_snprintf( buff, sizeof( buff), "%d'%d\"%d", h,m,sec);
    SetWindowText( GetDlgItem( s->hwnd, IDC_EDITSENDINFO2), buff);
    earth_conv_rad_to_hms( incoming_top->latitude, &h, &m, &sec);
    vce_snprintf( buff, sizeof( buff), "%d'%d\"%d", h,m,sec);
    SetWindowText( GetDlgItem( s->hwnd, IDC_EDITSENDINFO3), buff);
    ShowWindow( s->hwnd, SW_SHOW);
    s->serial = incoming_top->serial;
    /* free incoming_top */
    incoming_list_t *bak;
    bak = incoming_top->next;
    free( incoming_top);
    incoming_top = bak;
    if( incoming_top == NULL) incoming_bottom = NULL;
    
    return 0;
}
void incoming_finalize( void)
{
    incoming_list_t *l, *bak;
    for( l = incoming_top; l != NULL; l = bak) {
        bak = l->next;
        free( l);
    }
}
incoming_list_t *incoming_get_top( void)
{
    return incoming_top;
}
