#define _INPUT_CPP_
#include <dinput.h>
#include <windows.h>

#include "input.h"
#include "common.h"
#include "main.h"
#include "win.h"

LPDIRECTINPUT8 d_input = NULL;
LPDIRECTINPUTDEVICE8 input_mouse = NULL;
LPDIRECTINPUTDEVICE8 input_keyboard = NULL;
DIPROPDWORD dipdw;


/* -------------------------------------------------------------------------
 * initialize mouse
 * -------------------------------------------------------------------------*/
BOOL input_mouse_init( void)
{
    if( FAILED( d_input->CreateDevice(
        GUID_SysMouse, &input_mouse, NULL)))
    {
        return FALSE;
    }        
    if( FAILED( input_mouse->SetDataFormat( &c_dfDIMouse))) {
        return FALSE;
    }
    if( FAILED( input_mouse->SetCooperativeLevel(
        hWnd,
        DISCL_NONEXCLUSIVE|DISCL_FOREGROUND)))
    {
        return FALSE;
    }
//    input_mouse_event = CreateEvent( NULL, FALSE, FALSE, NULL);
//    if( input_mouse_event == NULL) return FALSE;

//    if( FAILED( input_mouse->SetEventNotification( input_mouse_event))) {
//        return FALSE;
//    }
    dipdw.diph.dwSize = sizeof( DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof( DIPROPHEADER);
    dipdw.diph.dwObj = 0;
    dipdw.diph.dwHow = DIPH_DEVICE;
    dipdw.dwData = INPUT_MOUSE_BUFF_MAX;

    if( FAILED( input_mouse->SetProperty( DIPROP_BUFFERSIZE, &dipdw.diph))) {
        return FALSE;
    }
    input_mouse_set_acquire();
    ZeroMemory( input_keyboard_data, sizeof( input_keyboard_data));
    return TRUE;
}    
/* -------------------------------------------------------------------------
 * initialize keyboard
 * -------------------------------------------------------------------------*/
BOOL input_keyboard_init( void)
{
    if( FAILED(
        d_input->CreateDevice(GUID_SysKeyboard, &input_keyboard, NULL)))
    {
        return FALSE;
    }
    if( FAILED(
        input_keyboard->SetDataFormat( &c_dfDIKeyboard)))
    {
        return FALSE;
    }
    if( FAILED(
        input_keyboard->SetCooperativeLevel(
            hWnd,
            DISCL_NONEXCLUSIVE|DISCL_FOREGROUND)))
    {
        return FALSE;
    }
    input_keyboard->Acquire();
            
        
                                         
    return TRUE;
}    
/* -------------------------------------------------------------------------
 * initialize input
 * -------------------------------------------------------------------------*/
BOOL input_init( void)
{
    if( FAILED( DirectInput8Create(
        hInst, DIRECTINPUT_VERSION,
        IID_IDirectInput8, ( void **)&d_input, NULL)))
    {
        return FALSE;
    }
    /* initialize mouse */
    if( !input_mouse_init()) return FALSE;
    /* initialize keyboard */
    if( !input_keyboard_init()) return FALSE;
    return TRUE;
}
/* -------------------------------------------------------------------------
 * finalize input
 * -------------------------------------------------------------------------*/
void input_finalize( void)
{
    if( input_mouse != NULL) {
        input_mouse->Unacquire();
        input_mouse->Release();
    }
    if( input_keyboard != NULL) {
        input_keyboard->Unacquire();
        input_keyboard->Release();
    }
    if( d_input != NULL) {
        d_input->Release();
    }
}
/* -------------------------------------------------------------------------
 * It gets or releases accessing permition of mouse device
 *  depending on the window state.
 * -------------------------------------------------------------------------*/
void input_mouse_set_acquire( void)
{
    if( input_mouse == NULL) return;
    if( gActive) {
        input_mouse->Acquire();
    }
    else {
        input_mouse->Unacquire();
    }
}                        
/* -------------------------------------------------------------------------
 * It gets or releases access permition of keyboard device.
 * -------------------------------------------------------------------------*/
void input_keyboard_set_acquire( void)
{
    if( input_keyboard == NULL) return;
    input_keyboard->Acquire();
}                        
/* -------------------------------------------------------------------------
 * loop
 * -------------------------------------------------------------------------*/
void input_mouse_loop( void)
{
    DIDEVICEOBJECTDATA dev_obj[INPUT_MOUSE_BUFF_MAX];
    HRESULT result = WAIT_TIMEOUT;
    DWORD elem = INPUT_MOUSE_BUFF_MAX;
    unsigned int i;
    POINT mouse_pos;
    input_mouse_move_x = 0;
    input_mouse_move_y = 0;
    input_mouse_rbutton_new_pressed = FALSE;
    input_mouse_lbutton_new_pressed = FALSE;
    input_mouse_move_flg_x = FALSE;
    input_mouse_move_flg_y = FALSE;
    /* It gets mouse event when window is only active. */
    if( gActive) {
        /* It gets information from mouse device. */
        result = input_mouse->GetDeviceData( sizeof( DIDEVICEOBJECTDATA),
                                             dev_obj, &elem, 0);
        if( result == DIERR_INPUTLOST){
            input_mouse_set_acquire();
            return;
        }
        if( FAILED( result) || elem == 0 ) {
            return;
        }
        for( i = 0; i < elem; i ++ ) {
            switch( dev_obj[i].dwOfs) {
            case DIMOFS_X:
                input_mouse_move_x += dev_obj[i].dwData;
                input_mouse_move_flg_x = TRUE;
                break;
            case DIMOFS_Y:
                input_mouse_move_y += dev_obj[i].dwData;
                input_mouse_move_flg_y = TRUE;
                break;
            case DIMOFS_BUTTON1: /* right button */
                if( dev_obj[i].dwData & 0x80) {
                    if( !input_mouse_rbutton) {
                        input_mouse_rbutton_new_pressed = TRUE;
                    }
                    input_mouse_rbutton = TRUE;
                }
                else {
                    input_mouse_rbutton = FALSE;
                }
                break;
            case DIMOFS_BUTTON0: /* left button */
                if( dev_obj[i].dwData & 0x80) {
                    if( !input_mouse_lbutton ) {
                        input_mouse_lbutton_new_pressed = TRUE;
                    }
                    input_mouse_lbutton = TRUE;
                }
                else {
                    input_mouse_lbutton = FALSE;
                }
                break;
            }
        }
        if( input_mouse_move_flg_x || input_mouse_move_flg_y ) {
            GetCursorPos( &mouse_pos);
            ScreenToClient( hWnd, &mouse_pos);
            mouse_x = mouse_pos.x;
            mouse_y = mouse_pos.y;
            if( screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST) {
                game_end = TRUE;
            }
        }
    }
}
/* -------------------------------------------------------------------------
 * loop
 * -------------------------------------------------------------------------*/
void input_keyboard_loop( void)
{
    char prev[256];
    int i;
    memcpy( prev, input_keyboard_data, sizeof( prev));
    if( FAILED(
        input_keyboard->GetDeviceState(
            sizeof( input_keyboard_data), (LPVOID)&input_keyboard_data)))
    {
        ZeroMemory( input_keyboard_data, sizeof( input_keyboard_data));
        while( input_keyboard->Acquire() == DIERR_INPUTLOST);
        return;
    }
    for( i = 0; i < 256; i ++ ) {
        input_keyboard_data_new_pressed[i] = (prev[i] ^ input_keyboard_data[i]) & input_keyboard_data[i];
    }
    if( screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST) {
        for( i = 0; i < 256; i ++ ) {
            if( input_keyboard_data[i]) {
                game_end = TRUE;
                break;
            }
        }
    }
}
