#define _M_CPP_

/*
  m.cpp : Math subroutines
  
*/

#include "vce.h"
#include <math.h>
#include <d3d8.h>
#include <d3dx8.h>

#include "m.h"
#include "game_main.h"
#include "common.h"
#include "draw.h"

int sign( float a, float b )
{
    if( a >= b ) return 1; else return 0;
}


float sphere_distance( float x1, float y1, float x2, float y2, float h )
{
    D3DXVECTOR3 v1,v2;
    earth_get_v_lolah( &v1, x1,y1, h );
    earth_get_v_lolah( &v2, x2,y2, h );

    float dx = v2.x - v1.x;
    float dy = v2.y - v1.y;
    float dz = v2.z - v1.z;
    return (float) sqrt( dx * dx + dy * dy + dz * dz );
}
float distance( float x1, float y1, float x2, float y2 )
{
    float dx = x2 - x1;
    float dy = y2 - y1;
    return (float) sqrt( dx * dx + dy * dy  );
}

/* Make a random-lined sphere */
D3DXVECTOR3 random_sphere_vec( D3DXVECTOR3 vec, float radius )
{
    float dx,dy,dz;
    float u = ((float)( rand() % 100 )/100.0f) * 2.0f * PI;
    float v = ((float)( rand() % 100 )/100.0f) * 2.0f * PI;

    dx = radius * cosf(u) * cosf(v);
    dy = radius * sinf(u);
    dz = radius * cosf(u) * sinf(v);

    vec.x += dx;
    vec.y += dy;
    vec.z += dz;
    return vec;
}

/* lo,la : -180 to 180  h_km : kilo meter */
void earth_get_v_lolah( D3DXVECTOR3 *v, float lo, float la, float h_km )
{
    float lo_rad = lo / 180.0f * PI - PI/2.0f;
    float la_rad = la / 180.0f * PI;
    float h = 0.64f + h_km / 1000.0f;
    
    v->x = h * cosf(la_rad) * cosf(lo_rad);
    v->y = h * sinf(la_rad);
    v->z = h * cosf(la_rad) * sinf(lo_rad);
}
void earth_get_lolah( float x, float y, float z, float *lo, float *la)
{
    float t;
    *la = (float)asin( y) * 180.0f / PI;
    /* normalize zx axis */
    t = 1.0f / (float)sqrt((x * x + z * z));
    z *= t;
    *lo = (float)asin( z);
    *lo += PI / 2.0f;
    if( x < 0.0f ) *lo *= -1.0f;
    *lo *= 180.0f/PI;
}
void earth_conv_rad_to_hms( float r, int *h, int *min, int *sec)
{
    float w;
    *h = (int)r;
    w = (int)((r - (int)r)*60.0f);
    if( w < 0.0f ) w *= -1.0f;
    *min = w;
    *sec = (int)((w - (int)w)* 60.0f);
}
int calc_cursor_intersect( int x, int y, D3DXVECTOR3 *ret)
{
//    int x = mouse_x;
//    int y = mouse_y;
    
    D3DXVECTOR3 v;
    D3DXMATRIX mat_proj;
    d3d_dev->GetTransform( D3DTS_PROJECTION, &mat_proj );
        
    v.x = ((( 2.0f * x ) /  client_area_width ) - 1 ) / mat_proj._11;
    v.y = - ((( 2.0f * y ) /  client_area_height ) - 1 ) / mat_proj._22;
    v.z = 1.0f;

        // Get the inverse view matrix
    D3DXMATRIX mat_view, m;
    d3d_dev->GetTransform( D3DTS_VIEW, &mat_view );
    D3DXMatrixInverse( &m, NULL, &mat_view );

    D3DXVECTOR3 pick_ray_v, pick_ray_orig_v;

    pick_ray_v.x  = v.x*m._11 + v.y*m._21 + v.z*m._31; // a
    pick_ray_v.y  = v.x*m._12 + v.y*m._22 + v.z*m._32; // b
    pick_ray_v.z  = v.x*m._13 + v.y*m._23 + v.z*m._33; // c
    pick_ray_orig_v.x = m._41; /* A */
    pick_ray_orig_v.y = m._42; /* B */
    pick_ray_orig_v.z = m._43; /* C */
    /* calculate t */
    /* (a^2 + b^2 + c^2)t^2 + 2(aA + bB + cC)t + (A^2 + B^2 + C^2 - r^2) = 0 */
    /*         a                   b                    c                    */
    /* t = ( -b +- sqrt( b^2 - 4ac)) /2a */
    float t;
    float a,b,c;
    a = pick_ray_v.x * pick_ray_v.x
        + pick_ray_v.y * pick_ray_v.y
        + pick_ray_v.z * pick_ray_v.z;
    b = 2.0f*( pick_ray_v.x * pick_ray_orig_v.x
            + pick_ray_v.y * pick_ray_orig_v.y
            + pick_ray_v.z * pick_ray_orig_v.z);
    c = pick_ray_orig_v.x * pick_ray_orig_v.x
        + pick_ray_orig_v.y * pick_ray_orig_v.y
        + pick_ray_orig_v.z * pick_ray_orig_v.z
        - 1.0f;
    if( b * b - 4.0f*a*c < 0 ) return 0;
    t = (-b -(float)sqrt(b*b-4.0f*a*c))/(2.0f*a);
    ret->x = ( pick_ray_orig_v.x + t * pick_ray_v.x);
    ret->y = ( pick_ray_orig_v.y + t * pick_ray_v.y);
    ret->z = ( pick_ray_orig_v.z + t * pick_ray_v.z);
    return 1;
}
