#define _MAIN_CPP_
#include "vce.h"

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <d3d8.h>
#include <d3dx8.h>

#include "main.h"
#include "win.h"
#include "draw.h"
#include "state.h"
#include "mesh.h"
#include "obj.h"
#include "input.h"
#include "frame.h"
#include "font.h"
#include "resource.h"
#include "config.h"
#include "parsecmd.h"
#include "msgdlg.h"
#include "incoming.h"

#define NO_DRAW_MAX_COUNT 20
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrev, char *CmdLine, int CmdShow)
{
    MSG msg;
    HACCEL accelerator_handle;
    hInst = hInstance;
    parse_command_line( GetCommandLine());
    config_read();
    if( screen_saver_mode == SMODE_CONFIG) {
        hWnd_config_dlg = create_config( hwnd_scr_parent, 1);
        if( (int)hwnd_scr_parent != 0 ) return 0; // modal dialog box
        /* for config dialogbox message loop */
        while( 1) {
            if( hWnd_config_dlg) {
                if( GetMessage( &msg, NULL, 0, 0)) {
                    /* for modeless propertysheet */
                    if( PropSheet_GetCurrentPageHwnd( hWnd_config_dlg) == NULL) {
                        DestroyWindow( hWnd_config_dlg);
                        hWnd_config_dlg = NULL;
                    }
                    else PropSheet_IsDialogMessage( hWnd_config_dlg, &msg);
                }
            }
            else break;
        }
        return 0;
    }
    else if( screen_saver_mode == SMODE_PREVIEW) {
        /* PENDING */
        return 0;
    }
    if( !window_init()) {
        MessageBox( hWnd, "failed: initialize window.",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    if( !d3d_init()) {
        MessageBox( hWnd, "failed: initialize direct x",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    if( !input_init()) {
        MessageBox( hWnd, "failed: initialize direct input.",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    if( !mesh_init()) {
        MessageBox( hWnd, "failed: read meshes.",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    if( !font_init()) {
        MessageBox( hWnd, "failed: initialize font.",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    if( !msgdlg_init()) {
        MessageBox( hWnd, "failed: initialize msgdlg.",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    if( !incoming_init()) {
        MessageBox( hWnd, "failed: initialize incoming.",
                    EARTH_WINDOWNAME, MB_OK |MB_ICONSTOP);
        return 0;
    }
    srand( time(NULL));
    game_end = FALSE;
    no_draw_count = 0;
    frame_initialize();
    frame_set_sec_start();
    draw_polygon_set_sec_start();
    accelerator_handle = LoadAccelerators( hInst,
                                           MAKEINTRESOURCE( IDR_ACCELERATOR1));
    if( accelerator_handle == NULL) {
        MessageBox( hWnd, "failed: initialize accelerator",
                    EARTH_WINDOWNAME, MB_OK|MB_ICONSTOP);
        return 0;
    }
    hWnd_config_dlg = NULL;
    hWnd_version_dlg = NULL;
    if( screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST ) {
        while( ShowCursor( FALSE) >= 0);
    }
    /* game's main loop */
    while( !game_end) {
        DWORD proc_time_start;
        DWORD draw_time_start;
        proc_time_start = GetTickCount();
        font_reset();
        input_mouse_loop();    /* process mouse input */
        input_keyboard_loop();  /* process keyboard input */
        /* message proc */
        while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE)) {
            int is_dlg = FALSE;
            if( IsDialogMessage( hWnd_version_dlg, &msg)) {
                is_dlg = TRUE;
            }
            else {
                msg_dlg_t *s;
                ARRAY_SCAN_TYPE( msg_dlg_mi, s, msg_dlg_t *) {
                    if( IsDialogMessage( s->hwnd, &msg)) {
                        is_dlg = TRUE;
                        break;
                    }
                }
            }
            if( !is_dlg && hWnd_config_dlg) {
                /* for modeless propertysheet */
                if( PropSheet_GetCurrentPageHwnd( hWnd_config_dlg) == NULL) {
                    DestroyWindow( hWnd_config_dlg);
                    hWnd_config_dlg = NULL;
                }
                else if( PropSheet_IsDialogMessage( hWnd_config_dlg, &msg)) {
                    is_dlg = TRUE;
                }
            }
            if( !is_dlg) {
                if( !TranslateAccelerator( hWnd, accelerator_handle, &msg)) {
                    TranslateMessage( &msg);
                    DispatchMessage( &msg);
                }
            }
        }
        state_loop();
        frame_calc_frame_rate();
        draw_calc_polygon_rate();
        current_proc_time = GetTickCount() - proc_time_start;
        /* Is processing less than time of one frame(It is defined frame.cpp)?
         * If so,next refresh time will be set in frame_is_refresh_time().
         */
        if( !frame_is_refresh_time()) {
            /* not draw when no_draw_count lower than NO_DRAW_MAX_COUNT.
             * draw force when it overed.
             */
            if( no_draw_count < NO_DRAW_MAX_COUNT) {
                no_draw_count ++;
                continue;
            }
        }
        draw_time_start = GetTickCount();
        draw_main();
        current_draw_time = GetTickCount() - draw_time_start;
        /* sleep when remained time */
        frame_sleep();
        if( no_draw_count >= NO_DRAW_MAX_COUNT) {
            frame_reset();
        }
        else {
            frame_set_next_refresh();
        }
        no_draw_count = 0;
        frame_count ++;
    }
    if( screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST ) {
        while( ShowCursor( TRUE) < 0);
    }
    incoming_finalize();
    font_finalize();
    obj_finalize();
    input_finalize();
    mesh_finalize();
    d3d_finalize();
    config_write();
    return 0;
}
