#include <stdio.h>
#include "mesh.h"
#include "common.h"
#include "draw.h"

mesh_t *mesh;
int mesh_num;

BOOL mesh_init( void)
{
    LPDIRECT3DDEVICE8 dev;
    LPD3DXBUFFER adjacency_buffer;
    
    unsigned int num = 0;
    unsigned int num2 = 0;
    unsigned int i;
    WIN32_FIND_DATA find_data;
    HANDLE file_handle;
    
    mesh = NULL;
    mesh_num = 0;
    
    /* read data in directory. */
    file_handle = FindFirstFile( "*.x", &find_data);
    if( file_handle == INVALID_HANDLE_VALUE) return TRUE;
    num ++;
    while( 1 ) {
        if( FindNextFile( file_handle, &find_data) == 0 ) break;
        /* count file num for do malloc */
        num ++;
    }
    
    dev = d3d_get_d3d_dev();
    if( dev == NULL) return FALSE;
    
    /* allocate memory */
    mesh = (mesh_t *)malloc( sizeof( mesh_t) * num);
    if( mesh == NULL) return FALSE;
    for( i = 0; i < num; i ++) {
        (mesh + i)->id = -1;
        (mesh + i)->num = 0;
        (mesh + i)->materials = NULL;
        (mesh + i)->textures = NULL;
        (mesh + i)->mesh = NULL;
        
    }
    file_handle = FindFirstFile( "*.x", &find_data);
    if( file_handle == INVALID_HANDLE_VALUE) return TRUE;
    while( 1 ) {
        int len;
        DWORD material_num;
        LPD3DXBUFFER material_buffer;
        LPD3DXMESH mesh_work;
        D3DXMATERIAL *materials;
        
        if( FAILED( D3DXLoadMeshFromX( find_data.cFileName,
//                                       D3DXMESH_SYSTEMMEM,
                                       D3DXMESH_MANAGED,
                                       dev,
                                       &adjacency_buffer,
                                       &material_buffer,
                                       &material_num,
                                       &mesh_work)))
        {
            continue;
        }
        if( FAILED( mesh_work->OptimizeInplace(
            D3DXMESHOPT_COMPACT|D3DXMESHOPT_ATTRSORT|D3DXMESHOPT_VERTEXCACHE,
            (DWORD*)adjacency_buffer->GetBufferPointer(),
            NULL, NULL, NULL)))
        {
            mesh_work->Release();
            material_buffer->Release();
            adjacency_buffer->Release();
            continue;
        }
        len = strlen( find_data.cFileName);
        find_data.cFileName[len-2] = '\0';
        
        materials = (D3DXMATERIAL *)material_buffer->GetBufferPointer();
        /* allocate memory for material */
        ( mesh + num2)->materials =
            (D3DMATERIAL8 *)malloc( sizeof( D3DMATERIAL8) * material_num);
        if( ( mesh + num2)->materials == NULL) return FALSE;
        /* allocate memory for texture */
        ( mesh + num2)->textures =
            (LPDIRECT3DTEXTURE8 *)malloc( sizeof( LPDIRECT3DTEXTURE8) * material_num);
        if( ( mesh + num2)->textures == NULL) return FALSE;
        for( i = 0; i < material_num; i ++ ) {
            HRESULT ret;
            (mesh + num2)->materials[i] = materials[i].MatD3D;
            (mesh + num2)->materials[i].Ambient = (mesh + num2)->materials[i].Diffuse;
            /* texture file open */
            ret = D3DXCreateTextureFromFile( dev,
                                                   materials[i].pTextureFilename,
                                             &(mesh + num2)->textures[i]);
            if( FAILED( ret)) {
                (mesh + num2)->textures[i] = NULL;
            }
        }
        (mesh + num2)->mesh = mesh_work;
        (mesh + num2)->num = material_num;
        (mesh + num2)->id = atoi( find_data.cFileName);
        material_buffer->Release();
        adjacency_buffer->Release();
        num2 ++;
        if( num2 > num) break;
        if( FindNextFile( file_handle, &find_data) == 0 ) break;
    }
    mesh_num = num2;
    
    
    return TRUE;
    
}
void mesh_finalize( void)
{
    int i, j;
    for( i = 0; i < mesh_num; i ++ ) {
        free( (mesh + i)->materials);
        for( j = 0; j < (mesh + i)->num; j ++ ) {
            if( (mesh + i)->textures[j] != NULL) {
                (mesh + i)->textures[j]->Release();
            }
        }
        if( (mesh + i)->textures != NULL) {
            free( (mesh + i)->textures);
        }
        (mesh + i)->mesh->Release();
    }
}    
mesh_t *mesh_get_mesh_from_id( int id)
{
    int i;
    for( i = 0; i < mesh_num; i ++ ) {
        if( (mesh + i)->id == id ) return (mesh + i);
    }
    return NULL;
}
