/*
  net.cpp : Connect to Luminousphere server whenever available


*/


#include "vce.h"
#include "net.h"
#include "lumiproto_cli.h"
#include "obj.h"
#include "drawearth.h"
#include "drawupoint.h"
#include "drawmsg.h"
#include "win.h"
#include <time.h>
#include "config.h"
#include "log.h"
#include "msg.h"

tcpcontext_t tcp = NULL;
conn_t co;
int user_num;

NETSTATE netstat = NET_INIT;
time_t netstat_time;
/* This indicates the position is not decided yet */
int my_longitude = INIT_LONGITUDE; 
int my_latitude;
unsigned int my_serial = 0;

void set_netstat( NETSTATE s )
{
    netstat_time = time(NULL);
    netstat = s;
}

int net_init( void )
{
    vce_initialize();
    tcp = vce_tcpcontext_create( 0,
                                 NULL, 0,
                                 1,
                                 65536, 2048,
                                 LUMIPROTO_BIG_TIME_OUT,
                                 VCE_BLOCK_CIPHER_NONE, 0,
                                 1, 0,0 );
    if(!tcp) return 0;

    vce_tcpcontext_set_conn_parser( tcp,
                                    vce_protocol_parser_bin16,
                                    lumiproto_cli_pcallback );
    set_netstat( NET_INIT );
    return 1;
}
void net_poll( void )
{
    vce_heartbeat();

    switch( netstat ){
    case NET_INIT:
        if( my_longitude != INIT_LONGITUDE ){        
            co = vce_tcpcontext_connect( tcp, config.hostname.str,
                                         config.port.int_val);
            set_netstat( NET_CONNECT);
            window_menu_update();
            user_num = 0;
        }
        break;
    case NET_CONNECT:
        if( time(NULL) > netstat_time + 5 ){ /* retry in 5 secs */
            set_netstat(NET_INIT);
            vce_conn_close( co);
            vce_conn_clear( &co);
            break;
        }
        if( vce_conn_writable( co ) > 0 ){
            OutputDebugString( "Network OK!\n");
            set_netstat(NET_ESTABLISHED);
            break;
        }
        break;
    case NET_ESTABLISHED:
        if( time(NULL) > netstat_time + 5 ){ /* wait for login 5 secs */
            set_netstat(NET_INIT);
            vce_conn_close( co);
            vce_conn_clear(&co);
            break;
        }
        OutputDebugString( "Sending login\n" );
        int majv, minv;
        majv = lumiproto_cli_get_version( &minv );
        OutputDebugString( "sending ping\n");
        lumiproto_ping_send( co, majv, minv,
                             GetTickCount(), 1000 );
        lumiproto_login_user_send( co,
                                   config.nickname.str,
                                   strlen( config.nickname.str),
                                   my_longitude,
                                   my_latitude,
                                   config.color_r.int_val,
                                   config.color_g.int_val,
                                   config.color_b.int_val);
        set_netstat(NET_WAITLOGIN);
        break;
    case NET_WAITLOGIN:
        if( time(NULL) > netstat_time + 5 ){ /* wait for login 5 secs */
            set_netstat(NET_INIT);
            vce_conn_close( co);
            vce_conn_clear(&co);
            break;
        }
        break;
    case NET_LOGIN:
        if( vce_conn_writable( co ) <= 0 ){
            OutputDebugString( "Network down!\n");
            set_netstat(NET_INIT);
        }
        if( time(NULL) > netstat_time + 120 ){ /* ping in 120 secs */
            int majv, minv;
            majv = lumiproto_cli_get_version( &minv );
            OutputDebugString( "sending ping\n");
            lumiproto_ping_send( co, majv, minv,
                                 GetTickCount(), 1000 );
//            set_netstat(NET_ESTABLISHED); /* put it again */
            set_netstat(NET_LOGIN); /* put it again */
        }
        
        break;
    }
}
int net_logout( void)
{
    vce_conn_close( co);
    netstat = NET_INIT;
    my_longitude = INIT_LONGITUDE;
    upoint_allclear();
    user_num = 0;
    window_menu_update();
    return 0;
}

int net_sendmsg( unsigned int serial, char *str)
{
    if( vce_conn_writable( co ) > 0 ){
        lumiproto_msg_send( co, serial, str, strlen( str));
        return 0;
    }    
    return -1;
}
int net_announce( char *pass, char *msg)
{
    if( vce_conn_writable( co ) > 0 ){
        lumiproto_announce_send( co, pass, strlen( pass), msg, strlen( msg), 255,255,0);
        return 0;
    }
    return -1;
}
int lumiproto_pong_recv( conn_t _c, int majv, int minv, int tick )
{
    int my_majv, my_minv;
    my_majv = lumiproto_cli_get_version( &my_minv );
    if( majv != my_majv || minv != my_minv) {
        message_append( "This client version is old.", 255,255,0);
        message_append( "Please get the newset version from LuminouSphere Web Site.", 255,255,0);
        message_append( "http://www.luminousphere.net/", 255,255,0);
    }
    return 0;
}

int lumiproto_login_user_r_recv( conn_t _c, int result, int serial )
{
    if( result == 0 ){
        OutputDebugString( "Login OK\n" );
        set_netstat(NET_LOGIN);
        my_serial = (unsigned)serial;
    } else {
        OutputDebugString( "Login fail\n");
        net_logout();
//        set_netstat(NET_INIT);
        return -1;
    }
    return 0;
}

int lumiproto_user_info_pasv_recv( conn_t _c, int serial,
                                   char *name, int name_len,
                                   int x, int y, int status,
                                   char r, char g, char b )
{
    /* Receiving new user, put new user on the screen */
    float lo = ((float)x) / (float)LUMIPROTO_FIX_POINT;
    float la = ((float)y) / (float)LUMIPROTO_FIX_POINT;
    obj_t *eo;
    char n[LUMIPROTO_NAME_MAX+1];
    vce_makecstr( n, sizeof( n), name, name_len);
    
    eo = obj_search( "earth" );
    if( !upoint_init( (unsigned)serial, n, lo, la, 500.0f, eo, r,g,b ) ){
        return 0;
    }
    OutputDebugString( "Put new user on the earth\n" );
    user_num ++;
    return 0;
}

int lumiproto_user_remove_pasv_recv( conn_t _c, int serial )
{
//    float lo = ((float)x) / (float)LUMIPROTO_FIX_POINT;
//    float la = ((float)y) / (float)LUMIPROTO_FIX_POINT;
    upoint_clear( serial );
    user_num --;
    return 0;
}

int lumiproto_msg_pasv_recv( conn_t _c, int from_serial,
                             char *text, int text_len )
{
    char buff[LUMIPROTO_MSG_MAX+1];
    vce_makecstr( buff, sizeof( buff), text, text_len);
    log_write( from_serial, buff);
    obj_t *from_o = find_upoint_from_serial( (unsigned)from_serial );
    obj_t *to_o  = find_upoint_from_serial( my_serial);
    if( from_o && to_o ){
        obj_t *eo = obj_search( "earth" );
        
        msg_init( from_o, to_o, eo, buff, 1, ((upoint_work_t *)from_o->work)->name );
    }
    return 0;
}

int lumiproto_msg_watch_recv( conn_t _c, int from_serial, int to_serial,
                              char r, char g, char b )
{
//    float flo = ((float)from_x) / (float)LUMIPROTO_FIX_POINT;
//    float fla = ((float)from_y) / (float)LUMIPROTO_FIX_POINT;
//    float tlo = ((float)to_x) / (float)LUMIPROTO_FIX_POINT;
//    float tla = ((float)to_y) / (float)LUMIPROTO_FIX_POINT;    
    obj_t *from_o = find_upoint_from_serial( (unsigned)from_serial);
    obj_t *to_o = find_upoint_from_serial( (unsigned)to_serial);

    if( from_o && to_o ){
        obj_t *eo = obj_search( "earth" );
        msg_init( from_o, to_o, eo, "", 0,((upoint_work_t *)from_o->work)->name);
    }

    return 0;
}
int lumiproto_announce_pasv_recv( conn_t _c, char *msg, int msg_len, char r, char g, char b )
{
    char buff[LUMIPROTO_MSG_MAX+1];
    vce_makecstr( buff, sizeof( buff), msg, msg_len);
    message_append( buff, (int)r,(int)g,(int)b);
    return 0;
}

int lumiproto_cli_sender( conn_t _c, char *d, int l )
{
    return vce_protocol_unparser_bin16( _c, d,l );
}
    
int lumiproto_cli_recv_error_callback( conn_t c, int e )
{
    OutputDebugString( "Protocol error\n");
    return -1;
}
