#ifndef _OBJ_H__
#define _OBJ_H__

#include <windows.h>
#include "common.h"
#include <d3d8.h>
#include <d3dx8.h>

typedef enum {
    OBJ_TYPEUPOINT,
    OBJ_TYPENUM
}OBJ_TYPE;

// object (link list)
typedef struct _obj_t
{
    OBJ_TYPE type;
    int seqnum;                       // object id
    int priority;                     // priority of linking list(by ascending)
    int visible;
    float x;                         // x coordinate in logical
    float y;                         // y coordinate in logical
    float z;
    float dx, dy, dz;
    float rotate_x,rotate_y,rotate_z;
    float d_rotate_x,d_rotate_y,d_rotate_z;
    char name[64];
    int mesh_id;
    int angle;                          /*  ~ 360 */
    double scale_x;                     
    double scale_y;                     
    int draw_priority;
    BOOL destroy_flg;
    void (*motionfunc)( struct _obj_t *);
    void (*drawfunc)( struct _obj_t *);   // call when rendering
    void (*destroyfunc)( struct _obj_t *);
    void *work;                       // working memory
    struct _obj_t *next;          // next pointer
    struct _obj_t *prev;          // previous pointer

    struct _obj_t *parent;        //

} obj_t;


# ifdef _OBJ_CPP_
#  define OBJ_EXTERN
# else
#  define OBJ_EXTERN extern
# endif

OBJ_EXTERN obj_t *obj_start;
OBJ_EXTERN int obj_num;


# if defined(__cplusplus)
extern "C" {
# endif

BOOL obj_initialize(void);
void obj_heartbeat( void);
obj_t *obj_new( int priority, int worksize, char *nm);
BOOL obj_delete(obj_t *obj);
void obj_delete_all(void);
void obj_reset(void);
void obj_finalize(void);
obj_t *obj_get_start( void);
obj_t *obj_search( char *name );
void obj_delete_by_id( char *s );
void obj_delete_by_type( OBJ_TYPE t);

# if defined(__cplusplus)
}  /* extern "C" { */
# endif


#endif
