#define _WIN_CPP_
#include "vce.h"
#include <windows.h>
#include <stdio.h>
#include "win.h"
#include "main.h"
#include "input.h"
#include "draw.h"
#include "drawearth.h"
#include "resource.h"
#include "config.h"
#include "net.h"
#include "game_main.h"
#include "camera.h"
#include "incoming.h"
#include "camera.h"
#include "obj.h"
#include "drawupoint.h"
#include "announce.h"

LRESULT CALLBACK version_dlg_proc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
    switch( msg) {
    case WM_INITDIALOG:
        return FALSE;
    case WM_COMMAND:
        switch( LOWORD( wp)) {
        case IDOK:
            DestroyWindow( hwnd);
            hWnd_version_dlg = NULL;
        }
        break;
    case WM_CLOSE:
        DestroyWindow( hwnd);
        hWnd_version_dlg = NULL;
        break;
    default:
        return FALSE;        
    }
    return TRUE;
}
/* -------------------------------------------------------------------------
 * window callback function
 * -------------------------------------------------------------------------*/
LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
        RECT rect;
    switch( msg) {
    case WM_ACTIVATE:
        if( wParam == WA_INACTIVE) gActive = FALSE;
        else gActive = TRUE;
        /* acquire or release permission of access */
        input_mouse_set_acquire();
        input_keyboard_set_acquire();
        break;
    case WM_EXITSIZEMOVE:
        GetClientRect( hWnd, &rect);
        if( rect.right - rect.left != client_area_width ||
            rect.bottom - rect.top != client_area_height)
        {
            draw_restore();
        }
        return 0;
    case WM_COMMAND:
        switch( LOWORD( wParam)) {
        case ID_VIEW_WIREFRAME:
            config.view_wireframe.int_val = !config.view_wireframe.int_val;
            break;
        case ID_VIEW_ROTATE:
            config.view_rotate.int_val = !config.view_rotate.int_val;
            break;
        case ID_VIEW_FINDROUTE:
            config.view_findroute.int_val = !config.view_findroute.int_val;
            break;
        case ID_OPTION_CONFIG:
            if( !hWnd_config_dlg) {
                hWnd_config_dlg = create_config( hWnd, 0);
            }
            break;
        case ID_VIEW_RESETVIEWPOINT:
            camera_init();
            break;
        case ID_VIEW_VERBOSE:
            config.view_verbose.int_val = !config.view_verbose.int_val;
            break;
        case ID_FILE_LOGIN:
            login_user();
            break;
        case ID_FILE_LOGOUT:
            net_logout();
            break;
        case ID_VIEW_AUTOMOVEVIEWPOINT:
            config.view_automove_viewpoint.int_val = !config.view_automove_viewpoint.int_val;
            break;
        case ID_HELP_VERSION:
            if( !hWnd_version_dlg) {
                hWnd_version_dlg = CreateDialog(
                    hInst, MAKEINTRESOURCE( IDD_DIALOG_VERSION),
                    hWnd,(DLGPROC)version_dlg_proc);
                ShowWindow( hWnd_version_dlg, SW_SHOW);
            }
            break;
        case ID_HELP_WEB:
            ShellExecute( NULL, "open",
                          "http://www.luminousphere.net/", NULL, NULL, SW_SHOWNORMAL);
            break;
        case ID_SENDMESSAGE:
            select_upoint_mode = 1;
            break;
        case ID_INCOMINGMESSAGE:
        {
            obj_t *obj;
            upoint_work_t *up;
            incoming_list_t *l;
            l = incoming_get_top();
            if( l != NULL) {
                obj = find_upoint_from_serial( l->serial);
                if( obj != NULL) {
                    up = (upoint_work_t *)obj->work;
                    camera_move_to_upoint_init( up->longitude, up->latitude);
                    up->hit = 1;
                }
            }
            incoming_create_dialog();
            incoming_msg_num --;
            if( incoming_msg_num <= 0 ) {
                EnableMenuItem( hMenu, ID_INCOMINGMESSAGE, MF_GRAYED);
                /* If doesn't, Menu appearance isn't reflected immediately. */
                DrawMenuBar( hWnd);
                incoming_msg_num = 0;
            }
        }
        window_menu_update();
            break;
        case ID_ANNOUNCE:
            announce_create_dialog();
            break;
        }
        break;
    case WM_ACTIVATEAPP:
        if( wParam == FALSE &&
            (screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST))
        {
            game_end = TRUE;
        }
        break;
    case WM_POWERBROADCAST:
        if( wParam == PBT_APMSUSPEND &&
            (screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST))
        {
            game_end = TRUE;
        }
        break;
#if 0
    case WM_MOUSEMOVE:
    case WM_KEYDOWN:
    case WM_LBUTTONDOWN:
    case WM_RBUTTONDOWN:
        if( screen_saver_mode == SMODE_SCR || screen_saver_mode == SMODE_TEST) {
            game_end = TRUE;
        }
        break;
#endif
    case WM_CLOSE:
        if( !screen_saver_mode) {
            GetClientRect( hWnd, &rect);
            config.last_x.int_val = rect.left;
            config.last_y.int_val = rect.top;
            config.last_width.int_val = rect.right - rect.left;
            config.last_height.int_val = rect.bottom - rect.top;
        }
        game_end = TRUE;
        return 1;
    }
    return DefWindowProc( hWnd, msg, wParam, lParam);
}
/* -------------------------------------------------------------------------
 * initialize
 * -------------------------------------------------------------------------*/
int window_init( void)
{    
    WNDCLASS wc;
    ZeroMemory( &wc, sizeof( WNDCLASS));
    wc.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH);
    wc.hCursor = LoadCursor( NULL, IDC_ARROW);
    wc.hInstance = hInst;
    wc.lpfnWndProc = WndProc;
    wc.lpszClassName = EARTH_CLASSNAME;
    RegisterClass( &wc);
    DWORD dwStyle;
    RECT rc;
    switch( screen_saver_mode) {
    case SMODE_PREVIEW:
        GetClientRect( hwnd_scr_parent, &rc);
        dwStyle = WS_VISIBLE |WS_CHILD;
        window_width = rc.right - rc.left;
        window_height = rc.bottom - rc.top;
        hWnd = CreateWindow( EARTH_CLASSNAME, EARTH_WINDOWNAME,
                             dwStyle,
                             rc.left, rc.top,
                             window_width, window_height,
                             hwnd_scr_parent,
                             NULL, hInst, NULL);
        break;
        dwStyle = WS_VISIBLE;
        window_width = GetSystemMetrics( SM_CXSCREEN);
        window_height = GetSystemMetrics( SM_CYSCREEN);
        client_area_width = window_width;
        client_area_height = window_height;
        hWnd = CreateWindow( EARTH_CLASSNAME, EARTH_WINDOWNAME,
                             dwStyle,
                             0,0,
                             window_width, window_height,
                             hwnd_scr_parent,
                             NULL, hInst, NULL);
    case SMODE_TEST:
        break;
    case SMODE_SCR:
    default:
        window_width = config.last_width.int_val + GetSystemMetrics( SM_CXSIZEFRAME) * 2;
        
        window_height = config.last_height.int_val
            + GetSystemMetrics( SM_CYSIZEFRAME) * 2
            + GetSystemMetrics( SM_CYCAPTION)
            + GetSystemMetrics( SM_CYMENU);
        client_area_width = config.last_width.int_val;
        client_area_height = config.last_height.int_val;
        dwStyle = WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU
            |WS_THICKFRAME|WS_MINIMIZEBOX|WS_VISIBLE;
#ifdef _DEBUG 
        hMenu = LoadMenu( hInst, MAKEINTRESOURCE( IDR_MENU2));
#else
        hMenu = LoadMenu( hInst, MAKEINTRESOURCE( IDR_MENU1));
#endif
        hWnd = CreateWindow( EARTH_CLASSNAME, EARTH_WINDOWNAME,
                         dwStyle,
                         config.last_x.int_val,config.last_y.int_val,
                         window_width, window_height,
                         NULL,hMenu,hInst, NULL);
    }
    return TRUE;
}
/* -------------------------------------------------------------------------
 * update menu
 * -------------------------------------------------------------------------*/
void window_menu_update( void)
{
    CheckMenuItem( hMenu, ID_VIEW_WIREFRAME,
                   config.view_wireframe.int_val ? MF_CHECKED : MF_UNCHECKED);
    CheckMenuItem( hMenu, ID_VIEW_ROTATE,
                   config.view_rotate.int_val ? MF_CHECKED : MF_UNCHECKED);
    CheckMenuItem( hMenu, ID_VIEW_FINDROUTE,
                   config.view_findroute.int_val ? MF_CHECKED : MF_UNCHECKED);
    CheckMenuItem( hMenu, ID_VIEW_AUTOMOVEVIEWPOINT,
                   config.view_automove_viewpoint.int_val ? MF_CHECKED : MF_UNCHECKED);
    CheckMenuItem( hMenu, ID_VIEW_VERBOSE,
                   config.view_verbose.int_val ? MF_CHECKED : MF_UNCHECKED);
    if( my_longitude == INIT_LONGITUDE) {
        EnableMenuItem( hMenu, ID_FILE_LOGIN, MF_ENABLED);
        EnableMenuItem( hMenu, ID_FILE_LOGOUT, MF_GRAYED);
    }
    else {
        EnableMenuItem( hMenu, ID_FILE_LOGIN, MF_GRAYED);
        EnableMenuItem( hMenu, ID_FILE_LOGOUT, MF_ENABLED);
    }
}
