#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "vce.h"
#include "lumimain.h"
#include "lumiproto_sv.h"
#include "lumirecv.h"
#include "ut.h"
#include "user.h"


options_t opts =
{
    1, /* verbose */
    1, /* printd_out */
    39999, /* swp_port */
    3, /* swp_max */
    2, /* swp_hiwater */
    1024 * 1024 * 2, /* swp_rw_buf */
    4000, /* circ_max */
    3000, /* circ_hiwater */
    16384, /*  circ_rw_buf */
    3600, /* client_timeout */
    10, /* print_oneline_every */
    "log", /* errout_dir */
    39998,
    "hoge", /* announce_pass */
    "name.log", /* namelogfile */
    "lumisv.total.stat", /* statfile */
};
statistics_t stats;

/* runtime vars */
tcpcontext_t tcp_swp, tcp_adm;
unsigned int current_time;

/* -------------------------------------------------------------------------
 *
 * -----------------------------------------------------------------------*/
static int swp_accept_callback( conn_t c )
{
    char a[100];
    vce_errout( "lumisv: accepted connection from swp: %s\n",
                vce_conn_get_remote_ipv4_addr_string(c,a,sizeof(a)));
    stats.swp_accept_count += 1.0;
    return 0;
    
}
/* -------------------------------------------------------------------------
 *
 * -----------------------------------------------------------------------*/
static int swp_hiwater_accept_callback( conn_t c, int num )
{
    char a[100];
    vce_errout( "lumisv: (hiwater) accepted connection from swp: %s\n",
                vce_conn_get_remote_ipv4_addr_string(c,a,sizeof(a)));
    return 0;
}

/* -------------------------------------------------------------------------
 *
 * -----------------------------------------------------------------------*/
static int swp_close_callback( conn_t c, CLOSE_REASON reason )
{
    char a[100];
    vce_errout( "lumisv: closeed connection from swp: %s\n",
                vce_conn_get_remote_ipv4_addr_string(c,a,sizeof(a)));
    return 0;    
}
/* -------------------------------------------------------------------------
 *
 * -----------------------------------------------------------------------*/
static int adm_pcallb( conn_t c, char *d, int l )
{
    char out[2048];
    stats.admin_stats_ok += 1.0f;
    make_stats_text( out, sizeof(out));
    vce_protocol_unparser_text( c, out, strlen(out));
    vce_conn_close(c);
    return 0;
}
static int adm_acceptw( conn_t c )
{
    char adr[1000];
    vce_conn_get_remote_addr_string( c, adr, sizeof(adr));
    vce_errout( "lumisv: admin: accepted from %s\n",adr );
    return 0;
}
/* -------------------------------------------------------------------------
 *
 * -----------------------------------------------------------------------*/
static int config_callback( char *f, int l, char *p, char *v )
{
    if( strcmp( "verbose", p ) == 0 ){
        opts.verbose = atoi(v);
    } else if( strcmp( "printd_out", p ) == 0 ){
        opts.printd_out = atoi(v);
    } else if( strcmp( "client_timeout",p)==0){
        opts.client_timeout = atoi(v);
    } else if( strcmp( "print_oneline_every",p)==0){
        opts.print_oneline_every = atoi(v);
    } else if( strcmp( "circ_rw_buf", p ) == 0 ){
        opts.circ_rw_buf = atoi(v);
    } else if( strcmp( "circ_max", p ) == 0 ){
        opts.circ_max = atoi(v);
    } else if( strcmp( "circ_hiwater", p ) == 0 ){
        opts.circ_hiwater = atoi(v);
    } else if( strcmp( "swp_max", p ) == 0 ){
        opts.swp_max = atoi(v);
    } else if( strcmp( "swp_hiwater", p ) == 0 ){
        opts.swp_hiwater = atoi(v);
    } else if( strcmp( "swp_rw_buf", p ) == 0 ){
        opts.swp_rw_buf = atoi(v);        
    } else if( strcmp( "swp_port", p ) == 0 ){
        opts.swp_port = atoi(v);
    } else if( strcmp( "errout_dir", p ) == 0 ){
        vce_copy_cstring( opts.errout_dir, sizeof(opts.errout_dir), v );
    } else if( strcmp( "admin_port", p ) == 0 ){
        opts.admin_port = atoi(v);
    } else if( strcmp( "announce_pass", p ) == 0 ){
        vce_copy_cstring( opts.announce_pass, sizeof(opts.announce_pass),v);
    } else if( strcmp( "namelog", p ) == 0 ){
        vce_copy_cstring( opts.namelogfile, sizeof(opts.namelogfile),v);
    } else if( strcmp( "statfile", p ) == 0 ){
        vce_copy_cstring( opts.statfile, sizeof(opts.statfile),v);
    } else {
        vce_errout( "lumisv: %s:%d: bad directive %s\n", f,l,p);
        return -1;
    }
    return 0;
}

static void save_stat( char *fn )
{
    char buf[1024];
    FILE *fp = fopen( fn, "w");
    if(!fp){
        return;
    }

    make_stats_text( buf, sizeof(buf));
    fprintf( fp, "%s",buf);
    fclose(fp);
}
static int statcb( char *f, int l, char *p, char *v )
{
    if( strcmp( "login_total", p ) == 0 ){
        stats.login_total = atof(v);
    } else if( strcmp( "msg_total", p ) == 0){
        stats.msg_total = atof(v);
    }
    return 0;
}
static void load_stat( char *fn )
{
    vce_read_simple_config_file( fn, statcb );
}



/* -------------------------------------------------------------------------
 *
 * -----------------------------------------------------------------------*/
int main( int argc, char **argv )
{
    int done = 0;
    int argi;
    if( argc < 2 ){
        vce_errout( "Usage: lumisv conf_flie conf_file ...\n");
        return 1;
    }

    printd_init( "lumisv", 1 );
    vce_initialize();

    vce_errout_config( opts.errout_dir, 1024 * 1024 * 100 );
    
    for(argi=1;argi<argc;argi++){
        int r;
        r = vce_read_simple_config_file( argv[argi], config_callback );
        if( r == VCE_EACCESSFILE ){
            vce_errout( "lumisv: file %s not found!\n", argv[argi] );
            return 1;
        } else if( r <0 ){
            vce_errout( "lumisv: error reading %s\n", argv[argi] );
        } else {
            vce_errout( "lumisv: read %s\n", argv[argi] );
        }
    }

    load_stat( opts.statfile );
    
    /* 10 : maintainance connections */
    user_init( opts.circ_max + 10 );
    
    /* VCE things */
    tcp_swp = vce_tcpcontext_create( 1, "0.0.0.0",
                                     opts.swp_port,
                                     opts.swp_max,
                                     opts.swp_rw_buf,
                                     opts.swp_rw_buf,
                                     LUMIPROTO_BIG_TIME_OUT,
                                     VCE_BLOCK_CIPHER_NONE, 0,
                                     0,0,
                                     sizeof(circ_opts) );

    if( !tcp_swp ){
        vce_errout( "lumisv: cannot create tcp_swp:%s", STRERR );
        return 1;
    }
    vce_tcpcontext_set_conn_parser( tcp_swp,
                                    vce_protocol_parser_vecmain_server,NULL);
    vce_tcpcontext_set_conn_acceptwatcher( tcp_swp,
                                           swp_accept_callback );
    vce_tcpcontext_set_conn_hiwater_acceptwatcher(
        tcp_swp, opts.swp_hiwater, swp_hiwater_accept_callback);
    vce_tcpcontext_set_conn_closewatcher( tcp_swp,
                                          swp_close_callback );
    vce_tcpcontext_set_circ_parser( tcp_swp,
                                    vce_vecprotocol_parser_bin16,
                                    lumiproto_sv_pcallback );
    vce_tcpcontext_set_circ_acceptwatcher( tcp_swp,
                                           lumiproto_acceptwatcher );
    vce_tcpcontext_set_circ_hiwater_acceptwatcher(
        tcp_swp, opts.circ_hiwater,
        lumiproto_hiwater_accept_callback);    
    vce_tcpcontext_set_circ_closewatcher( tcp_swp,
                                          lumiproto_closewatcher );

    if( vce_vec_init_mainserv( tcp_swp,
                               opts.circ_max,
                               opts.circ_rw_buf ) < 0 ){
        printd( "vce_vec_init_mainserv fail:%s", STRERR );
        vce_finalize();
        return 1;
    }

    /* Init admin */
    tcp_adm = vce_tcpcontext_create( 1, "0.0.0.0",
                                     opts.admin_port,
                                     4, 4096,4096,
                                     3600,
                                     VCE_BLOCK_CIPHER_NONE, 0,
                                     0,0,0 );
    if(!tcp_adm){
        vce_errout( "lumisv: cannot initialize admin tcpcontext\n" );
        vce_finalize();
        return 1;
    }
    vce_tcpcontext_set_conn_parser( tcp_adm,
                                    vce_protocol_parser_text,
                                    adm_pcallb );
    vce_tcpcontext_set_conn_acceptwatcher( tcp_adm,
                                           adm_acceptw );
        
                                     
    vce_set_heartbeat_wait_flag(1);
    vce_errout( "lumisv: start loop\n");
    while( !done ) {
        time_t nt = time(NULL);
        if( nt > (current_time+20) ){
            vce_errout( "lumisv: alive %u\n",nt);
            save_stat( opts.statfile );
            current_time = nt;
        }
        vce_heartbeat();
    }
    vce_finalize();
    vce_errout( "lumisv: finished\n" );
    return 0;
}


/* Make statistics text */
char *make_stats_text( char *out, int outlen )
{
    int majv, minv;
    majv = lumiproto_sv_get_version( &minv );
    
    vce_snprintf( out, outlen,
                  "accept=%.0f\n"
                  "hi_accept=%.0f\n"
                  "close=%.0f\n"
                  "login=%.0f\n"
                  "login_e=%.0f\n"
                  "send_status=%.0f\n"
                  "msg=%.0f\n"
                  "msg_watch=%.0f\n"                  
                  "ping=%.0f\n"
                  "too_long_msg=%.0f\n"
                  "msg_user_not_found=%.0f\n"
                  "msg_user_found=%.0f\n"                  
                  "swp_accept=%.0f\n"
                  "admin_stats_ok=%.0f\n"
                  "concurrent=%d\n"
                  "login_total=%.0f\n"
                  "msg_total=%.0f\n"
                  "major_version=%d\n"
                  "minor_version=%d\n"
                  "end\n"
                  ,
                  stats.accept_count,
                  stats.hi_accept_count,
                  stats.close_count,
                  stats.login_count,
                  stats.login_e_count,
                  stats.send_status_count,
                  stats.msg_count,
                  stats.msg_watch_count,
                  stats.ping_count,
                  stats.too_long_msg_count,
                  stats.msg_user_not_found_count,
                  stats.msg_user_found_count,
                  stats.swp_accept_count,
                  stats.admin_stats_ok,
                  vce_get_array_usenum( user_mi ),
                  stats.login_total,
                  stats.msg_total,
                  majv, minv
                  );
    return out;
}



void put_namelog( char *n )
{
    FILE *fp;
    fp = fopen( opts.namelogfile, "a+" );
    if(!fp){
        return;
    }
    fprintf( fp, "time=%u name='%s'\n", (unsigned int) time(NULL), n );
    fclose(fp);
}
