/*
  erecv.c

*/

#include <string.h>

#include "vce.h"
#include "lumiproto_sv.h"
#include "ut.h"
#include "lumimain.h"
#include "lumirecv.h"
#include "user.h"


#define INIT_CIRC_OPT circ_opts *copt = vce_circ_get_state_buffer(_c,NULL)


#define CHECK_HIWATER_RETURN(msg) if( copt->hiwater ){ printd( "HIWATER(%s)",(msg)); return -1; }



int lumiproto_sv_recv_error_callback( circ_t c,int e )
{
    printd( "detected protocol error: %d", e  );
    return -1;
}


int lumiproto_sv_sender( circ_t c, char *data, int len )
{
    return vce_vecprotocol_unparser_bin16( c,data,len);
}

int lumiproto_acceptwatcher( circ_t _c )
{
    char a[100];
    INIT_CIRC_OPT;
    
    printd( "accept connection from client %s",
            vce_circ_get_remote_ipv4_addr_string( _c, a,sizeof(a)));

    /* initialize circ_opts */
    copt->hiwater = 0;
    stats.accept_count += 1.0;
    vce_circ_set_timeout( _c, opts.client_timeout );
    return 0;
}
/* special accept watcher for hi-load situation */
int lumiproto_hiwater_accept_callback( circ_t _c, int num )
{
    char a[100];
    INIT_CIRC_OPT;
    
    printd( "hiwater accept connection from client %s",
            vce_circ_get_remote_ipv4_addr_string( _c, a,sizeof(a)));

    copt->hiwater = 1;  /* no service will be available when hiwater */

    stats.hi_accept_count += 1.0;
    vce_circ_set_timeout( _c, 5 );
    return 0;
}
int lumiproto_closewatcher( circ_t _c, CLOSE_REASON reason )
{
    char a[100];
    INIT_CIRC_OPT;
    printd( "closed connection from client %s reason=%d",
            vce_circ_get_remote_ipv4_addr_string( _c, a,sizeof(a)),
            reason );
    stats.close_count += 1.0;
                             
    if( copt->u ){
        tell_user_logout_to_all( copt->u );
        user_free(copt->u);
    }
    
    return 0;            
}

/****************************************************************
  Definition of protocol receiver callback functions
 ****************************************************************/

int lumiproto_ping_recv( circ_t _c, int majv, int minv, int tick,
                         int cliversion )
{
    int sub,maj;
    
    printd( "received ping. %d,%d,%d,%d", majv,minv,tick,cliversion);

    maj = lumiproto_sv_get_version( &sub );
    lumiproto_pong_send( _c, maj, sub, tick );
    stats.ping_count += 1.0f;
    return 0;
}

int lumiproto_login_user_recv( circ_t _c, char *name, int name_len,
                               int x, int y, char r, char g, char b )
{
    char hoge[1000];
    user *nu;
    INIT_CIRC_OPT;
    char n[LUMIPROTO_NAME_MAX+1], escaped_n[LUMIPROTO_NAME_MAX*2+1];
    vce_makecstr( n, sizeof( n), name, name_len);
    vce_escape_string( escaped_n, sizeof(escaped_n), name, name_len );

    printd( "received login_user. name=%s from=%s x=%d y=%d",
            n,
            vce_circ_get_remote_addr_string( _c, hoge, sizeof(hoge)),
            x,y);

    put_namelog( escaped_n );

    if(copt->u){
        printd( "already login (%d,%d)!", x,y);
        return lumiproto_login_user_r_send( _c, -2, 0 );
    }
    if( x < -180*LUMIPROTO_FIX_POINT || y < -90*LUMIPROTO_FIX_POINT
        || x > 180*LUMIPROTO_FIX_POINT || y > 90*LUMIPROTO_FIX_POINT ){
        printd( "bad coordination (%d,%d)!", x,y);
        return lumiproto_login_user_r_send( _c, -1 ,0);
    }
    nu = user_alloc( _c, n, x,y,r,g,b );
    if(!nu){
        stats.login_e_count += 1.0;
        lumiproto_login_user_r_send( _c, -1,0 );
        return 0;
    }


    stats.login_count += 1.0;
    stats.login_total += 1.0;
    copt->u = nu;
    lumiproto_login_user_r_send( _c, 0, nu->serial );
    user_know_who_are_online(copt->u);
    tell_user_login_to_all(nu);
    return 0;
}


int lumiproto_msg_recv( circ_t _c, int serial, char *text, int text_len )
{
    INIT_CIRC_OPT;

    if( copt->u == NULL ){
        /* You have to login before sending message */
        return 0;
    }
    
    send_msg_user( copt->u->serial,
                   serial,
                   text, text_len,
                   copt->u->r, copt->u->g, copt->u->b
                   );
    return 0;
}



/* Announce to all users */
int lumiproto_announce_recv( circ_t _c, char *password, int password_len,
                             char *msg, int msg_len, char r, char g, char b )
{
    char escaped[4096];
    INIT_CIRC_OPT;
    if( copt->u == NULL ){
        /* You have to login before sending message */
        return 0;
    }    
    vce_escape_string( escaped, sizeof(escaped), msg, msg_len );
    vce_errout( "lumisv: announce. msg='%s'\n", escaped );
    send_announce_all_users( copt->u->serial, msg, msg_len, r,g,b );
    return 0;
}
