/*
  user.c : handle,search user instances

  $Revision: 1.9 $
  $Date: 2002/08/07 15:34:12 $
  $Id: user.c,v 1.9 2002/08/07 15:34:12 ringo Exp $
  
 */

#include "lumimain.h"
#include "vce.h"
#include "user.h"
#include "ut.h"
#include "lumiproto_sv.h"
#include <string.h>


int user_mi=-1;

/* Simply use VCE array to handle user instances */
int user_init( int max )
{
    user_mi = vce_init_array( sizeof(user), max, "user_a" );
    if( user_mi < 0 ){
        printd( "cannot init user array\n");
        return user_mi;
    }
    return 0;
}

user *user_alloc( circ_t oc, char *n, int x, int y, char r, char g, char b )
{
    static unsigned int serial = 0;
    user *nu = (user*) vce_alloc_array_object( user_mi );
    if(!nu){
        printd( "cannot allocate user");
        return NULL;
    }
    memset( nu, 0, sizeof(*nu));
    nu->x = x;
    nu->y = y;
    nu->r = r;
    nu->g = g;
    nu->b = b;
    nu->when_allocate = time(NULL);
    nu->owner_circ = oc;
    nu->serial = serial++;
    vce_snprintf( nu->name, sizeof( nu->name), "%s", n);
    printd( "user_alloc: %d users available",
            vce_get_array_maxnum(user_mi) - vce_get_array_usenum(user_mi));

    return nu;
}
void tell_user_login_to_all(user *u )
{
    user *cur;
    if(!u)return;
    ARRAY_SCAN( user_mi, cur ){
        lumiproto_user_info_pasv_send( cur->owner_circ,
                                       u->serial, u->name, strlen( u->name),
                                       u->x, u->y,
                                       0,
                                       u->r, u->g, u->b );
    }
}
void tell_user_logout_to_all(user *u)
{
    user *cur;
    if(!u)return;
    ARRAY_SCAN( user_mi, cur ){
        if( u == cur ) continue;
        lumiproto_user_remove_pasv_send( cur->owner_circ, u->serial );
    }
}

void user_free( user *u )
{
    vce_free_array_object( user_mi, u );
}

#if 0
/* Tell one's new status to all other users  */
void notify_all_user_new_status( int x, int y, int status,
                                 char r, char g, char b)
{
    user *cur;
    ARRAY_SCAN( user_mi, cur ){
        lumiproto_user_info_pasv_send( cur->owner_circ,
                                       x, y, status,r,g,b );
    }
}
#endif

/* send message */
void send_msg_user( unsigned int from_serial, unsigned int to_serial, char *tex, int texlen,
                    char r, char g, char b )
{
    user *cur;
    stats.msg_count += 1.0;
    stats.msg_total += 1.0;    
    if( texlen >= LUMIPROTO_MSG_MAX ){
        stats.too_long_msg_count += 1.0;
        return;
    }
    ARRAY_SCAN( user_mi, cur ){
//        if( cur->x == to_x && cur->y == to_y ){
        if( cur->serial == to_serial) {
            lumiproto_msg_pasv_send( cur->owner_circ,
                                     from_serial,
                                     tex, texlen );
            stats.msg_user_found_count += 1.0;
            /* When sent, lets notify this all. */
            ARRAY_SCAN( user_mi, cur ){
                if( cur->serial != to_serial) {
                    lumiproto_msg_watch_send( cur->owner_circ,
                                              from_serial,
                                              to_serial, r,g,b );
                    stats.msg_watch_count += 1.0f;
                }
            }
            return;
        }
    }
    stats.msg_user_not_found_count += 1.0;
}

/* tell the user who are now online */
void user_know_who_are_online( user *u )
{
    user *cur;
    ARRAY_SCAN( user_mi, cur ){
        if( cur != u ){
            lumiproto_user_info_pasv_send( u->owner_circ,
                                           cur->serial,
                                           cur->name,
                                           strlen( cur->name),
                                           cur->x, cur->y,
                                           0,
                                           cur->r, cur->g, cur->b );
        }
    }
}
/* Send a message to all users */
void send_announce_all_users( unsigned int from_serial,
                              char *msg, int msg_len,
                              char r, char g, char b )
{
    user *cur;
    ARRAY_SCAN( user_mi, cur ){
        lumiproto_announce_pasv_send( cur->owner_circ, msg, msg_len, r,g,b );
        lumiproto_msg_watch_send( cur->owner_circ,
                                  from_serial, cur->serial,
                                  r,g,b );
    }
}
