/*
  et.c : Luminousphere test client

  $Revision: 1.9 $
  $Date: 2002/08/07 09:35:06 $
  $Id: lumitest.c,v 1.9 2002/08/07 09:35:06 ringo Exp $
  
 */

#include "vce.h"
#include "lumiproto_cli.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/time.h>

tcpcontext_t tcp;
conn_t c;
int sent_login = 0;
int sent_stats = 0;

typedef struct _friend_t
{
    int active;
    unsigned int serial;
    int x,y;
    char r,g,b;
} friend_t;

friend_t f[10000];

unsigned int my_serial;

/* Send messages to all friends! */
void sendtoall(void)
{
    int i;
    for(i=0;i<ELEMENTOF(f);i++){
        if(f[i].active){
            char buff[64];
            vce_snprintf( buff, sizeof( buff), "hello %d", rand()%1000);
            lumiproto_msg_send( c, f[i].serial, buff, strlen(buff) );
        }
    }
}
void newfriend(unsigned int serial, int x,int y, char r,char g,char b )
{
    int i;
    for(i=0;i<ELEMENTOF(f);i++){
        if( f[i].active == 1
            && f[i].x == x
            && f[i].y == y ){
            break;
        }
    }
    for(i=0;i<ELEMENTOF(f);i++){
        if(f[i].active == 0 ){
            f[i].x = x;
            f[i].y = y;
            f[i].r = r;
            f[i].g = g;
            f[i].b = b;
            f[i].active = 1;
            f[i].serial = serial;
            vce_errout( "new friend is coming.\n");
            break;
        }
    }
}
void removefriend( unsigned int serial)
{
    int i;
    for(i = 0;i<ELEMENTOF(f);i++){
//        if( f[i].active && f[i].x == x && f[i].y == y ){
        if( f[i].serial == serial ){
            f[i].active = 0;
            break;
        }
    }
}


int main( int argc, char **argv )
{
    char *host = NULL;
    int longtitude=0, latitude=0;
    if( argc < 4 ){
        vce_errout(  "usage: lumitest host longtitude latitude\n"  );
        return 1;
    }
    host = argv[1];
    longtitude = atoi(argv[2]);
    latitude = atoi(argv[3]);
    
    vce_initialize();

    tcp = vce_tcpcontext_create( 0, NULL, 0, 1,
                                 8192, 2048,
                                 365 * 24 * 60 * 60,
                                 VCE_BLOCK_CIPHER_NONE, 0,
                                 0, 0, 0 );
    if(!tcp){
        vce_errout( "et: cannot init tcpcontext\n");
        return 1;
    }
    vce_tcpcontext_set_conn_parser( tcp,
                                    vce_protocol_parser_bin16,
                                    lumiproto_cli_pcallback );

    c = vce_tcpcontext_connect( tcp, host,
                                LUMIPROTO_DEFAULT_FRONTEND_PORT );

    {
        struct timeval tm;
        gettimeofday( &tm, NULL );
        srand( tm.tv_usec );
    }

    vce_set_heartbeat_wait_flag(1);
    while(1){
        static time_t ahotime;
        vce_heartbeat();
        if( time(NULL) > ahotime ){
            ahotime = time(NULL);// + rand() % 8;
            sendtoall();
        }
    
        if( vce_conn_writable(c) <= 0 ){
            break;
        }

        if( sent_login == 0 ){
            sent_login = 1;
            vce_errout( "send login (%d,%d)\n", longtitude, latitude );
            lumiproto_login_user_send( c,
                                       longtitude,
                                       latitude,
                                       rand() % 256,
                                       rand() % 256,
                                       rand() % 256 );
        }
    }
    return 0;
}

int lumiproto_pong_recv( conn_t _c, int majv, int minv, int tick )
{
    return 0;
}

int lumiproto_login_user_r_recv( conn_t _c, int result, int serial )
{
    vce_errout( "login ok (%d)\n", result );
    my_serial = (unsigned)serial;
    return 0;
}

int lumiproto_user_info_pasv_recv( conn_t _c, int serial, int x, int y, int status,
                                   char r ,char g, char b)
{
    vce_errout( "user_info. (%d,%d) %d,%d,%d\n",x,y,r,g,b);
    if( (unsigned)serial == my_serial) return 0;
    newfriend( (unsigned)serial,x,y, r,g,b );
    return 0;
}
int lumiproto_user_remove_pasv_recv( conn_t _c, int serial)
{
    vce_errout( "user_remove. (%d)\n", serial );
    removefriend((unsigned)serial);
}

int lumiproto_msg_pasv_recv( conn_t _c, int seral,
                          char *text, int text_len )
{
/*    vce_errout( "msg. from (%d,%d) tl=%d\n",
      from_x, from_y, text_len ); */
    vce_errout( "m" );
    return 0;
}
        
int lumiproto_msg_watch_recv( conn_t _c, int from_serial, int to_serial,
                              char r, char g, char b )
{
/*    vce_errout( "msg_watch. (%d,%d) -> (%d,%d) %d,%d,%d\n",
      from_x, from_y, to_x, to_y, r,g,b ); */
    vce_errout( "W");
    return 0;
}


int lumiproto_cli_recv_error_callback( conn_t _c, int e )
{
    vce_errout( "et: protocol error.\n");
    return -1;    
}
int lumiproto_cli_sender( conn_t _c, char *data, int len )
{
    return vce_protocol_unparser_bin16( _c, data, len );
}
